#!/usr/bin/env python3
###
### Script to check water conservation in the IPSL coupled model
###
##  Warning, to install, configure, run, use any of included software or
##  to read the associated documentation you'll need at least one (1)
##  brain in a reasonably working order. Lack of this implement will
##  void any warranties (either express or implied).  Authors assumes
##  no responsability for errors, omissions, data loss, or any other
##  consequences caused directly or indirectly by the usage of his
##  software by incorrectly or partially configured personal
##
##
# SVN Information
SVN = {
    'Author'  : "$Author$",
    'Date'    : "$Date$",
    'Revision': "$Revision$",
    'Id'      : "$Id: ATM_waterbudget.py 6508 2023-06-13 10:58:38Z omamce $",
    'HeadURL' : "$HeadUrl: svn+ssh://omamce@forge.ipsl.jussieu.fr/ipsl/forge/projets/igcmg/svn/TOOLS/WATER_BUDGET/ATM_waterbudget.py $"
    }

###
## Import system modules
import sys, os, shutil#, subprocess, platform
import configparser, re

## Import needed scientific modules
import numpy as np, xarray as xr

# Check python version
if sys.version_info < (3, 8, 0) :
    print ( f'Python version : {platform.python_version()}' ) 
    raise Exception ( "Minimum Python version is 3.8" )

## Import local modules
import WaterUtils as wu
import libIGCM_sys
import nemo, lmdz

from WaterUtils import VarInt, Rho, Ra, Grav, ICE_rho_ice, ICE_rho_sno, OCE_rho_liq, ATM_rho, SRF_rho, RUN_rho, ICE_rho_pnd, YearLength

## Creates parser for reading .ini input file
## -------------------------------------------
config = configparser.ConfigParser ( interpolation=configparser.ExtendedInterpolation() )
config.optionxform = str # To keep capitals

## Experiment parameters
## ---------------------
ATM=None ; ATM_HIS='latlon' ; SRF_HIS='latlon' ; RUN_HIS='latlon' ; ORCA=None ; NEMO=None ; OCE_relax=False
OCE_icb=False ; Coupled=False ; Routing=None ; TestInterp=None
TarRestartPeriod_beg=None ; TarRestartPeriod_end=None ; Comment=None ; Period=None ; Title=None
YearBegin=None ; YearEnd=None ; DateBegin=None ; DateEnd=None

##
ARCHIVE=None ; STORAGE=None ; SCRATCHDIR=None ; R_IN=None ; rebuild=None ; TmpDir=None
FileDir=None ; FileOut=None
dir_ATM_his=None ; dir_SRF_his=None ; dir_OCE_his=None ; dir_ICE_his=None
FileCommon=None ; file_ATM_his=None ; file_SRF_his=None ; file_RUN_his=None
file_OCE_his=None ;  file_ICE_his=None ; file_OCE_sca=None
tar_restart_beg=None ; tar_restart_end=None ; file_ATM_beg=None ; file_ATM_end=None ; file_DYN_beg=None
file_DYN_end=None ; file_SRF_beg=None ; file_SRF_end=None
file_RUN_beg=None ; file_RUN_end=None ; file_RUN_end=None ; file_OCE_beg=None
file_ICE_beg=None ; file_OCE_beg=None
file_OCE_end=None ; file_ICE_beg=None ; file_OCE_end=None ; file_ICE_end=None
tar_restart_beg_ATM=None ; tar_restart_beg_DYN=None ; tar_restart_beg_SRF=None
tar_restart_beg_RUN=None ; tar_restart_beg_OCE=None ; tar_restart_beg_ICE=None
tar_restart_end_ATM=None ; tar_restart_end_DYN=None ; tar_restart_end_SRF=None
tar_restart_end_RUN=None ; tar_restart_end_OCE=None ; tar_restart_end_ICE=None
ContinueOnError=False ; ErrorCount=0 ; SortIco = False

##
## Precision of history file reading
## ---------------------------------
# Default is float (full precision). Degrade the precision by using np.float32
# Restart file are always read at the full precision
readPrec=float

## Read command line arguments
## ---------------------------
print ( "Name of Python script:", sys.argv[0] )
IniFile = sys.argv[1]

# Test existence of IniFile
if not os.path.exists (IniFile ) :
    raise FileExistsError ( f"File not found : {IniFile = }" )

if 'full' in IniFile : FullIniFile = IniFile
else                 : FullIniFile = 'full_' + IniFile
    
print ("Input file : ", IniFile )
config.read (IniFile)
FullIniFile = 'full_' + IniFile

## Reading config.card if possible
## -------------------------------
ConfigCard = None

if 'Experiment' in config.keys ()  : ## Read Experiment on Config file if possible
    if 'ConfigCard' in config['Experiment'].keys () :
        ConfigCard = config['Experiment']['ConfigCard']
        print ( f'{ConfigCard=}' )

if ConfigCard : ## Read config card if it exists
    # Text existence of ConfigCard
    if os.path.exists ( ConfigCard ) :
        print ( f'Reading Config Card : {ConfigCard}' )
        ## Creates parser for reading .ini input file
        MyReader = configparser.ConfigParser (interpolation=configparser.ExtendedInterpolation() )
        MyReader.optionxform = str # To keep capitals
        
        MyReader.read (ConfigCard)

        for VarName in ['JobName', 'ExperimentName', 'SpaceName', 'LongName', 'ModelName', 'TagName'] :
            if VarName in MyReader['UserChoices'].keys() :
                locals()[VarName] = MyReader['UserChoices'][VarName]
                exec  ( f'{VarName} = wu.setBool ({VarName})' )
                exec  ( f'{VarName} = wu.setNum  ({VarName})' )
                exec  ( f'{VarName} = wu.setNone ({VarName})' )
                exec  ( f'wu.{VarName} = {VarName}' )
                print ( f'    {VarName:21} set to : {locals()[VarName]:}' )
                
        for VarName in ['PackFrequency'] :
            if VarName in MyReader['Post'].keys() :
                locals()[VarName] = MyReader['Post'][VarName]
                exec  ( f'{VarName} = wu.setBool ({VarName})' )
                exec  ( f'{VarName} = wu.setNum  ({VarName})' )
                exec  ( f'{VarName} = wu.setNone ({VarName})' )
                exec  ( f'wu.{VarName} = {VarName}' )
                print ( f'    {VarName:21} set to : {locals()[VarName]:}' )
    else :
        raise FileExistsError ( f"File not found : {ConfigCard = }" )
                
## Reading config file
## -------------------
for Section in ['Config', 'Experiment', 'libIGCM', 'Files', 'Physics' ] :
   if Section in config.keys () : 
        print ( f'\nReading [{Section}]' )
        for VarName in config[Section].keys() :
            locals()[VarName] = config[Section][VarName]
            exec  ( f'{VarName} = wu.setBool ({VarName})' )
            exec  ( f'{VarName} = wu.setNum  ({VarName})' )
            exec  ( f'{VarName} = wu.setNone ({VarName})' )
            exec  ( f'wu.{VarName} = {VarName}' )
            print ( f'    {VarName:21} set to : {locals()[VarName]}' )
            #exec ( f'del {VarName}' )

print ( f'\nConfig file readed : {IniFile} ' )

##
## Reading prec
if wu.unDefined ( 'readPrec' )  :
    readPrec = np.float64
else :
    if readPrec in ["float", "float64", "r8", "double", "<class 'float'>"         ] : readPrec = float
    if readPrec in [         "float32", "r4", "single", "<class 'numpy.float32'>" ] : readPrec = np.float32
    if readPrec in [         "float16", "r2", "half"  , "<class 'numpy.float16'>" ] : readPrec = np.float16
    
## Some physical constants
## =======================
if wu.unDefined ( 'Ra' )           : Ra          = wu.Ra           #-- Earth Radius (m)
if wu.unDefined ( 'Grav' )         : Grav        = wu.Grav         #-- Gravity (m^2/s
if wu.unDefined ( 'ICE_rho_ice' )  : ICE_rho_ice = wu.ICE_rho_ice  #-- Ice volumic mass (kg/m3) in LIM3
if wu.unDefined ( 'ICE_rho_sno')   : ICE_rho_sno = wu.ICE_rho_sno  #-- Snow volumic mass (kg/m3) in LIM3
if wu.unDefined ( 'OCE_rho_liq' )  : OCE_rho_liq = wu.OCE_rho_liq  #-- Ocean water volumic mass (kg/m3) in NEMO
if wu.unDefined ( 'ATM_rho' )      : ATM_rho     = wu.ATM_rho      #-- Water volumic mass in atmosphere (kg/m^3)
if wu.unDefined ( 'SRF_rho' )      : SRF_rho     = wu.SRF_rho      #-- Water volumic mass in surface reservoir (kg/m^3)
if wu.unDefined ( 'RUN_rho' )      : RUN_rho     = wu.RUN_rho      #-- Water volumic mass of rivers (kg/m^3)
if wu.unDefined ( 'ICE_rho_pnd' )  : ICE_rho_pnd = wu.ICE_rho_pnd  #-- Water volumic mass in ice ponds (kg/m^3)
if wu.unDefined ( 'YearLength' )   : YearLength  = wu.YearLength   #-- Year length (s)
    
## Set libIGCM and machine dependant values
## ----------------------------------------
if not 'Files' in config.keys () : config['Files'] = {}

config['Physics'] = { 'Ra':str(Ra), 'Grav':str(Grav), 'ICE_rho_ice':str(ICE_rho_ice), 'ICE_rho_sno':str(ICE_rho_sno),
                      'OCE_rho_liq':str(OCE_rho_liq), 'ATM_rho':str(ATM_rho), 'SRF_rho':str(SRF_rho), 'RUN_rho':str(RUN_rho)}

config['Config'] = { 'ContinueOnError':str(ContinueOnError), 'SortIco':str(SortIco), 'TestInterp':str(TestInterp), 'readPrec':str(readPrec) }

## --------------------------
ICO  = ( 'ICO' in wu.ATM )
LMDZ = ( 'LMD' in wu.ATM )

mm = libIGCM_sys.config ( TagName=TagName, SpaceName=SpaceName, ExperimentName=ExperimentName, JobName=JobName, User=User, Group=Group,
             ARCHIVE=None, SCRATCHDIR=None, STORAGE=None, R_IN=None, R_OUT=None, R_FIG=None, rebuild=None, TmpDir=None, 
             R_SAVE=None, R_FIGR=None, R_BUFR=None, R_BUF_KSH=None, REBUILD_DIR=None, POST_DIR=None )
globals().update(mm)

config['Files']['TmpDir'] = TmpDir
config['libIGCM'] = { 'ARCHIVE':ARCHIVE, 'STORAGE':STORAGE, 'TmpDir':TmpDir, 'R_IN':R_IN, 'rebuild':rebuild }

## Defines begining and end of experiment
## --------------------------------------
if wu.unDefined ( 'DateBegin' ) :
    DateBegin = f'{YearBegin}0101'
    config['Experiment']['DateBegin'] = str(DateBegin)
else :
    YearBegin, MonthBegin, DayBegin = wu.SplitDate ( DateBegin )
    DateBegin = wu.FormatToGregorian (DateBegin)
    config['Experiment']['YearBegin'] = str(YearBegin)

if wu.unDefined ( 'DateEnd' )   :
    DateEnd   = f'{YearEnd}1231'
    config['Experiment']['DateEnd'] = str(DateEnd)
else :
    YearEnd, MonthEnd, DayEnd = wu.SplitDate ( DateEnd )
    DateEnd   = wu.FormatToGregorian (DateEnd)
    config['Experiment']['DateEnd'] = str(DateEnd)

if wu.unDefined ( 'PackFrequency' ) :
    PackFrequency = YearEnd - YearBegin + 1
    config['Experiment']['PackFrequency']   = f'{PackFrequency}'

if type ( PackFrequency ) == str : 
    if 'Y' in PackFrequency : PackFrequency = PackFrequency.replace ( 'Y', '') 
    if 'M' in PackFrequency : PackFrequency = PackFrequency.replace ( 'M', '')
    PackFrequency = int ( PackFrequency )
    
## Output file with water budget diagnostics
## -----------------------------------------
if wu.unDefined ( 'FileOut' ) :
    FileOut = f'ATM_waterbudget_{JobName}_{YearBegin}_{YearEnd}'
    if ICO : 
        if ATM_HIS == 'latlon' : FileOut = f'{FileOut}_LATLON'
        if ATM_HIS == 'ico'    : FileOut = f'{FileOut}_ICO'
    if readPrec == np.float32  : FileOut = f'{FileOut}_float32'
    FileOut = f'{FileOut}.out'

config['Files']['FileOut'] = FileOut

f_out = open ( FileOut, mode = 'w' )

## Useful functions
## ----------------
if readPrec == float :
    def rprec (tab) : return tab
else : 
    def rprec (tab) : return tab.astype(readPrec).astype(float)
    
def kg2Sv    (val, rho=ATM_rho) :
    '''From kg to Sverdrup'''
    return val/dtime_sec*1.0e-6/rho

def kg2myear (val, rho=ATM_rho) :
    '''From kg to m/year'''
    return val/ATM_aire_sea_tot/rho/NbYear

def var2prt (var, small=False, rho=ATM_rho) :
    if small :  return var , kg2Sv(var, rho=rho)*1000., kg2myear(var, rho=rho)*1000
    else     :  return var , kg2Sv(var, rho=rho)      , kg2myear(var, rho=rho)

def prtFlux (Desc, var, Form='F', small=False, rho=ATM_rho, width=15) :
    if small :
        if Form in ['f', 'F'] : ff=" {:14.6e} kg | {:12.4f} mSv | {:12.4f} mm/year "
        if Form in ['e', 'E'] : ff=" {:14.6e} kg | {:12.4e} mSv | {:12.4e} mm/year "
    else :
        if Form in ['f', 'F'] : ff=" {:14.6e} kg | {:12.4f} Sv  | {:12.4f} m/year  "
        if Form in ['e', 'E'] : ff=" {:14.6e} kg | {:12.4e} Sv  | {:12.4e} m/year  "
    echo ( (' {:>{width}} = ' +ff).format (Desc, *var2prt(var, small=small, rho=rho), width=width ) )
    return None

def echo (string, end='\n') :
    '''Function to print to stdout *and* output file'''
    print ( str(string), end=end  )
    sys.stdout.flush ()
    f_out.write ( str(string) + end )
    f_out.flush ()
    return None

echo ( f'{ContinueOnError = }' )
echo ( f'{SortIco         = }' )
echo ( f'{readPrec        = }' )

echo ( f'{JobName         = }' )
echo ( f'{ConfigCard      = }' )
echo ( f'{libIGCM         = }' )     
echo ( f'{User            = }' )       
echo ( f'{Group           = }' )       
echo ( f'{Freq            = }' )       
echo ( f'{YearBegin       = }' )     
echo ( f'{YearEnd         = }' )     
echo ( f'{DateBegin       = }' )
echo ( f'{DateEnd         = }' )
echo ( f'{PackFrequency   = }' ) 
echo ( f'{ATM             = }' )       
echo ( f'{Routing         = }' )       
echo ( f'{ORCA            = }' )      
echo ( f'{NEMO            = }' )      
echo ( f'{Coupled         = }' )      
echo ( f'{ATM_HIS         = }' )      
echo ( f'{SRF_HIS         = }' )      
echo ( f'{RUN_HIS         = }' )      

## Set libIGCM directories
## -----------------------
if wu.unDefined ('R_OUT'      ) : R_OUT       = os.path.join ( ARCHIVE   , 'IGCM_OUT' )
if wu.unDefined ('R_BUF'      ) : R_BUF       = os.path.join ( SCRATCHDIR, 'IGCM_OUT' )
if wu.unDefined ('L_EXP'      ) : L_EXP       = os.path.join (TagName, SpaceName, ExperimentName, JobName)
if wu.unDefined ('R_SAVE'     ) : R_SAVE      = os.path.join ( R_OUT, L_EXP )
if wu.unDefined ('R_BUFR'     ) : R_BUFR      = os.path.join ( R_BUF, L_EXP )
if wu.unDefined ('POST_DIR'   ) : POST_DIR    = os.path.join ( R_BUFR, 'Out' )
if wu.unDefined ('REBUILD_DIR') : REBUILD_DIR = os.path.join ( R_BUFR, 'REBUILD' )
if wu.unDefined ('R_BUF_KSH'  ) : R_BUF_KSH   = os.path.join ( R_BUFR, 'Out' )
if wu.unDefined ('R_FIGR'     ) : R_FIGR      = os.path.join ( STORAGE, 'IGCM_OUT', L_EXP )

config['libIGCM'].update ( { 'R_OUT':R_OUT, 'R_BUF':R_BUF, 'L_EXP':L_EXP, 'R_BUFR':R_BUFR, 'POST_DIR':POST_DIR,
                      'REBUILD_DIR':REBUILD_DIR, 'R_BUF_KSH':R_BUF_KSH, 'R_FIGR':R_FIGR, 'rebuild':rebuild } )

## Set directory to extract files
## ------------------------------
if wu.unDefined ( 'FileDir' ) : FileDir = os.path.join ( TmpDir, f'WATER_{JobName}' )
config['Files']['FileDir'] = FileDir

if not os.path.isdir ( FileDir ) : os.makedirs ( FileDir )

##- Set directories to rebuild ocean and ice restart files
if wu.unDefined ( 'FileDirOCE' ) : FileDirOCE = os.path.join ( FileDir, 'OCE' )
if wu.unDefined ( 'FileDirICE' ) : FileDirICE = os.path.join ( FileDir, 'ICE' )
if not os.path.exists ( FileDirOCE ) : os.mkdir ( FileDirOCE )
if not os.path.exists ( FileDirICE ) : os.mkdir ( FileDirICE )

echo (' ')
echo ( f'JobName     : {JobName}'    )
echo ( f'Comment     : {Comment}'    )
echo ( f'TmpDir      : {TmpDir}'     )
echo ( f'FileDir     : {FileDir}'    )
echo ( f'FileDirOCE  : {FileDirOCE}' )
echo ( f'FileDirICE  : {FileDirICE}' )

echo ( f'\nDealing with {L_EXP}'  )

## Creates model output directory names
## ------------------------------------
if Freq == "MO" : FreqDir = os.path.join ('Output' , 'MO' )
if Freq == "SE" : FreqDir = os.path.join ('Analyse', 'SE' )
if wu.unDefined ('dir_ATM_his' ) :
    dir_ATM_his = os.path.join ( R_SAVE, "ATM", FreqDir )
    config['Files']['dir_ATM_his'] = dir_ATM_his
if wu.unDefined ( 'dir_SRF_his' ) : 
    dir_SRF_his = os.path.join ( R_SAVE, "SRF", FreqDir )
    config['Files']['dir_SRF_his'] = dir_SRF_his
    
echo ( f'The analysis relies on files from the following model output directories : ' )
echo ( f'{dir_ATM_his = }' )
echo ( f'{dir_SRF_his = }' )

##-- Creates files names
if wu.unDefined ( 'Period' ) :
    if Freq == 'MO' : Period = f'{DateBegin}_{DateEnd}_1M'
    if Freq == 'SE' : Period = f'SE_{DateBegin}_{DateEnd}_1M'
    config['Files']['Period'] = Period

config['Files']['DateBegin'] = DateBegin
config['Files']['DateBegin'] = DateEnd
       
echo ( f'Period   : {Period}' )
    
if wu.unDefined ( 'FileCommon' ) :
    FileCommon = f'{JobName}_{Period}'
    config['Files']['FileCommon'] = FileCommon

if wu.unDefined ( 'Title' ) :
    Title = f'{JobName} : {Freq} : {DateBegin} - {DateEnd}'
    config['Files']['Title'] = Title
     
echo ('\nOpen history files' )
if wu.unDefined ( 'file_ATM_his' ) :
    if ATM_HIS == 'latlon' :
        file_ATM_his = os.path.join ( dir_ATM_his, f'{FileCommon}_histmth.nc' )
    if ATM_HIS == 'ico' :
        file_ATM_his = os.path.join ( dir_ATM_his, f'{FileCommon}_histmth_ico.nc' )
    config['Files']['file_ATM_his'] = file_ATM_his
if wu.unDefined ( 'file_SRF_his' ) :
    if ATM_HIS == 'latlon' :
        file_SRF_his = os.path.join ( dir_SRF_his, f'{FileCommon}_sechiba_history.nc' )
    if ATM_HIS == 'ico' :
        file_SRF_his = os.path.join ( dir_SRF_his, f'{FileCommon}_sechiba_history_ico.nc' )
    config['Files']['file_SRF_his'] = file_SRF_his

if Routing == 'SIMPLE' :
    if file_RUN_his == None :
        if ATM_HIS == 'latlon' :
            file_RUN_his = os.path.join ( dir_SRF_his, f'{FileCommon}_sechiba_history.nc' )
        if ATM_HIS == 'ico' :
            file_RUN_his = os.path.join ( dir_SRF_his, f'{FileCommon}_sechiba_history_ico.nc' )
        config['Files']['file_RUN_his'] = file_RUN_his

d_ATM_his = xr.open_dataset ( file_ATM_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
d_SRF_his = xr.open_dataset ( file_SRF_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
if Routing == 'SECHIBA' : d_RUN_his = d_SRF_his
if Routing == 'SIMPLE'  : d_RUN_his = xr.open_dataset ( file_RUN_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()

echo ( f'{file_ATM_his = }' )
echo ( f'{file_SRF_his = }' )
if Routing == 'SIMPLE' : echo ( f'{file_RUN_his = }' )

## Compute run length
## ------------------
dtime = ( d_ATM_his.time_counter_bounds.max() - d_ATM_his.time_counter_bounds.min() )
echo ('\nRun length : {:8.2f} days'.format ( (dtime/np.timedelta64(1, "D")).values ) )
dtime_sec = (dtime/np.timedelta64(1, "s")).values.item() # Convert in seconds

##-- Compute length of each period
dtime_per = (d_ATM_his.time_counter_bounds[:,-1] -  d_ATM_his.time_counter_bounds[:,0] )
echo ('\nPeriods lengths (days) : {:} days'.format ( (dtime_per/np.timedelta64(1, "D")).values ) )
dtime_per_sec = (dtime_per/np.timedelta64(1, "s")).values # In seconds
dtime_per_sec = xr.DataArray (dtime_per_sec, dims=["time_counter", ], coords=[d_ATM_his.time_counter,] )
dtime_per_sec.attrs['unit'] = 's'

##-- Number of years (approximative)
NbYear = dtime_sec / YearLength

##-- Extract restart files from tar

if wu.unDefined ('TarRestartDate_beg' ) : TarRestartDate_beg = wu.DateMinusOneDay ( DateBegin )
if wu.unDefined ('TarRestartDate_end' ) : TarRestartDate_end = wu.FormatToGregorian ( DateEnd )

if wu.unDefined ( 'TarRestartPeriod_beg' ) :

    TarRestartPeriod_beg_DateEnd = TarRestartDate_beg
    TarRestartPeriod_beg_DateBeg = wu.DateAddYear ( TarRestartPeriod_beg_DateEnd, -PackFrequency )
    TarRestartPeriod_beg_DateBeg = wu.DatePlusOneDay ( TarRestartPeriod_beg_DateBeg )
    
    TarRestartPeriod_beg = f'{TarRestartPeriod_beg_DateBeg}_{TarRestartPeriod_beg_DateEnd}'
    echo (f'Tar period for initial restart : {TarRestartPeriod_beg}')
    config['Files']['TarRestartPeriod_beg'] = TarRestartPeriod_beg

if wu.unDefined ( 'TarRestartPeriod_end' ) :

    TarRestartPeriod_end_DateEnd = TarRestartDate_end
    TarRestartPeriod_end_DateBeg = wu.DateAddYear ( TarRestartPeriod_end_DateEnd, -PackFrequency )
    TarRestartPeriod_end_DateBeg = wu.DatePlusOneDay ( TarRestartPeriod_end_DateBeg )
     
    TarRestartPeriod_end = f'{TarRestartPeriod_end_DateBeg}_{TarRestartPeriod_end_DateEnd}'
    echo (f'Tar period for final restart : {TarRestartPeriod_end}')
    config['Files']['TarRestartPeriod_end'] = TarRestartPeriod_end

echo (f'Restart dates - Start : {TarRestartPeriod_beg}  /  End : {TarRestartPeriod_end}')

if wu.unDefined ( 'tar_restart_beg' ) :
    tar_restart_beg = os.path.join ( R_SAVE, 'RESTART', f'{JobName}_{TarRestartPeriod_beg}_restart.tar' )
    config['Files']['tar_restart_beg'] = tar_restart_beg
if wu.unDefined ( 'tar_restart_end' ) :
    tar_restart_end = os.path.join ( R_SAVE, 'RESTART', f'{JobName}_{TarRestartPeriod_end}_restart.tar' )
    config['Files']['tar_restart_end'] = tar_restart_end
    
echo ( f'{tar_restart_beg = }' )
echo ( f'{tar_restart_end = }' )

##-- Names of tar files with restarts
if wu.unDefined ( 'SRF_HIS' ) : SRF_HIS = ATM_HIS

if wu.unDefined ( 'tar_restart_beg_ATM' ) : tar_restart_beg_ATM = tar_restart_beg
if wu.unDefined ( 'tar_restart_beg_DYN' ) : tar_restart_beg_DYN = tar_restart_beg
if wu.unDefined ( 'tar_restart_beg_SRF' ) : tar_restart_beg_SRF = tar_restart_beg
if wu.unDefined ( 'tar_restart_beg_RUN' ) : tar_restart_beg_RUN = tar_restart_beg
if wu.unDefined ( 'tar_restart_beg_OCE' ) : tar_restart_beg_OCE = tar_restart_beg
if wu.unDefined ( 'tar_restart_beg_ICE' ) : tar_restart_beg_ICE = tar_restart_beg

if wu.unDefined ( 'tar_restart_end_ATM' ) : tar_restart_end_ATM = tar_restart_end
if wu.unDefined ( 'tar_restart_end_DYN' ) : tar_restart_end_DYN = tar_restart_end
if wu.unDefined ( 'tar_restart_end_SRF' ) : tar_restart_end_SRF = tar_restart_end
if wu.unDefined ( 'tar_restart_end_RUN' ) : tar_restart_end_RUN = tar_restart_end
if wu.unDefined ( 'tar_restart_end_OCE' ) : tar_restart_end_OCE = tar_restart_end
if wu.unDefined ( 'tar_restart_end_ICE' ) : tar_restart_end_ICE = tar_restart_end

if wu.unDefined ( 'file_ATM_beg' ) : 
    file_ATM_beg = f'{FileDir}/ATM_{JobName}_{TarRestartDate_beg}_restartphy.nc'
    config['Files']['file_ATM_beg'] = file_ATM_beg
if wu.unDefined ( 'file_ATM_end' ) :
    file_ATM_end = f'{FileDir}/ATM_{JobName}_{TarRestartDate_end}_restartphy.nc'
    config['Files']['file_ATM_end'] = file_ATM_end

liste_beg = [file_ATM_beg, ]
liste_end = [file_ATM_end, ]

if wu.unDefined ( 'file_DYN_beg' ) :
    if LMDZ : file_DYN_beg = f'{FileDir}/ATM_{JobName}_{TarRestartDate_beg}_restart.nc'
    if ICO  : file_DYN_beg = f'{FileDir}/ICO_{JobName}_{TarRestartDate_beg}_restart.nc'
    liste_beg.append (file_DYN_beg)
    config['Files']['file_DYN_beg'] = file_DYN_beg
    
if wu.unDefined ( 'file_DYN_end' ) : 
    if LMDZ : file_DYN_end = f'{FileDir}/ATM_{JobName}_{TarRestartDate_end}_restart.nc'
    if ICO  : file_DYN_end = f'{FileDir}/ICO_{JobName}_{TarRestartDate_end}_restart.nc'
    liste_end.append ( file_DYN_end )
    config['Files']['file_DYN_end'] = file_DYN_end

if wu.unDefined ( 'file_SRF_beg' ) :
    file_SRF_beg = f'{FileDir}/SRF_{JobName}_{TarRestartDate_beg}_sechiba_rest.nc'
    config['Files']['file_SRF_beg'] = file_SRF_beg
if wu.unDefined ( 'file_SRF_end' ) :
    file_SRF_end = f'{FileDir}/SRF_{JobName}_{TarRestartDate_end}_sechiba_rest.nc'
    config['Files']['file_SRF_end'] = file_SRF_end
    
liste_beg.append ( file_SRF_beg )
liste_end.append ( file_SRF_end )

echo ( f'{file_ATM_beg = }')
echo ( f'{file_ATM_end = }')
echo ( f'{file_DYN_beg = }')
echo ( f'{file_DYN_end = }')
echo ( f'{file_SRF_beg = }')
echo ( f'{file_SRF_end = }')

if ICO :
    if wu.unDefined ('file_DYN_aire') : file_DYN_aire = os.path.join ( R_IN, 'ATM', 'GRID', ATM+'_grid.nc' )
    config['Files']['file_DYN_aire'] = file_DYN_aire

if Routing == 'SIMPLE' :
    if wu.unDefined ( 'file_RUN_beg' ) : 
        file_RUN_beg = f'{FileDir}/SRF_{JobName}_{TarRestartDate_beg}_routing_restart.nc'
        config['Files']['file_RUN_beg'] = file_RUN_beg
    if wu.unDefined ( 'file_RUN_end' ) : 
        file_RUN_end = f'{FileDir}/SRF_{JobName}_{TarRestartDate_end}_routing_restart.nc'
        config['Files']['file_RUN_end'] = file_RUN_end
        
    liste_beg.append ( file_RUN_beg )
    liste_end.append ( file_RUN_end )
    echo ( f'{file_RUN_beg = }' )
    echo ( f'{file_RUN_end = }' )

echo ('\nExtract restart files from tar : ATM, ICO and SRF')

def extract_and_rebuild ( file_name=file_ATM_beg, tar_restart=tar_restart_end, FileDirComp=FileDir, ErrorCount=ErrorCount ) :
    echo ( f'----------')
    echo ( f'file to extract : {file_name = }' )
    if os.path.exists ( os.path.join (FileDir, file_name) ) :
        echo ( f'file found : {file_name = }' )
    else :
        echo ( f'file not found : {file_name = }' )
        base_resFile = os.path.basename (file_name)
        if os.path.exists ( tar_restart ) :
            command =  f'cd {FileDir} ; tar xf {tar_restart} {base_resFile}'
            echo ( f'{command = }' )
            try : os.system ( command )
            except :
                if ContinueOnError :
                    ErrorCount += 1
                    echo ( f'****** Command failed : {command}' )
                    echo ( f'****** Trying to continue' )
                    echo ( ' ')
                else :
                    raise Exception ( f'**** command failed : {command} - Stopping' )
            else :
                echo ( f'tar done : {base_resFile}' )
        else :
            echo ( f'****** Tar restart file {tar_restart = } not found ' )
            if ContinueOnError :
                ErrorCount += 1
            else : 
                raise Exception ( f'****** tar file not found {tar_restart = } - Stopping' )
    return ErrorCount
 
ErrorCount += extract_and_rebuild ( file_name=file_ATM_beg, tar_restart=tar_restart_beg_ATM, FileDirComp=FileDir )
ErrorCount += extract_and_rebuild ( file_name=file_DYN_beg, tar_restart=tar_restart_beg_DYN, FileDirComp=FileDir )
ErrorCount += extract_and_rebuild ( file_name=file_SRF_beg, tar_restart=tar_restart_beg_SRF, FileDirComp=FileDir )

ErrorCount += extract_and_rebuild ( file_name=file_ATM_end, tar_restart=tar_restart_end_ATM, FileDirComp=FileDir )
ErrorCount += extract_and_rebuild ( file_name=file_DYN_end, tar_restart=tar_restart_end_DYN, FileDirComp=FileDir )
ErrorCount += extract_and_rebuild ( file_name=file_SRF_end, tar_restart=tar_restart_end_SRF, FileDirComp=FileDir )

if Routing == 'SIMPLE' :
    ErrorCount += extract_and_rebuild ( file_name=file_RUN_beg, tar_restart=tar_restart_beg_RUN, FileDirComp=FileDir )
    ErrorCount += extract_and_rebuild ( file_name=file_RUN_end, tar_restart=tar_restart_end_RUN, FileDirComp=FileDir )

##-- Exit in case of error in the opening file phase
if ErrorCount > 0 :
    echo ( '  ' )
    raise Exception ( f'**** Some files missing - Stopping - {ErrorCount = }' )

## 
echo ('\nOpening ATM SRF and ICO restart files')
d_ATM_beg = xr.open_dataset ( os.path.join (FileDir, file_ATM_beg), decode_times=False, decode_cf=True ).squeeze()
d_ATM_end = xr.open_dataset ( os.path.join (FileDir, file_ATM_end), decode_times=False, decode_cf=True ).squeeze()
d_SRF_beg = xr.open_dataset ( os.path.join (FileDir, file_SRF_beg), decode_times=False, decode_cf=True ).squeeze()
d_SRF_end = xr.open_dataset ( os.path.join (FileDir, file_SRF_end), decode_times=False, decode_cf=True ).squeeze()
d_DYN_beg = xr.open_dataset ( os.path.join (FileDir, file_DYN_beg), decode_times=False, decode_cf=True ).squeeze()
d_DYN_end = xr.open_dataset ( os.path.join (FileDir, file_DYN_end), decode_times=False, decode_cf=True ).squeeze()

for var in d_SRF_beg.variables :
    d_SRF_beg[var] = d_SRF_beg[var].where ( d_SRF_beg[var]<1.e20, 0.)
    d_SRF_end[var] = d_SRF_end[var].where ( d_SRF_end[var]<1.e20, 0.)

if Routing == 'SIMPLE' :
    d_RUN_beg = xr.open_dataset ( os.path.join (FileDir, file_RUN_beg), decode_times=False, decode_cf=True ).squeeze()
    d_RUN_end = xr.open_dataset ( os.path.join (FileDir, file_RUN_end), decode_times=False, decode_cf=True ).squeeze()

def to_cell ( dd, newname='cell' ) :
    '''Set space dimension to 'cell' '''
    for oldname in [ 'cell_mesh', 'y', 'points_physiques' ] :
        try    : dd = dd.rename ( {oldname : newname} )
        except : pass
    return dd

d_ATM_beg = to_cell ( d_ATM_beg )
d_ATM_end = to_cell ( d_ATM_end )
d_SRF_beg = to_cell ( d_SRF_beg )
d_SRF_end = to_cell ( d_SRF_end )
d_DYN_beg = to_cell ( d_DYN_beg )
d_DYN_end = to_cell ( d_DYN_end )

if Routing == 'SIMPLE' :
    d_RUN_beg = to_cell ( d_RUN_beg )
    d_RUN_end = to_cell ( d_RUN_end )

d_ATM_his = to_cell ( d_ATM_his )
d_SRF_his = to_cell ( d_SRF_his )
    
echo ( f'{file_ATM_beg = }' )
echo ( f'{file_ATM_end = }' )
echo ( f'{file_DYN_beg = }' )
echo ( f'{file_DYN_end = }' )
echo ( f'{file_SRF_beg = }' )
echo ( f'{file_SRF_end = }' )
if Routing == 'SIMPLE' :
    echo ( f'{file_RUN_beg = }' )
    echo ( f'{file_RUN_end = }' )

## Write the full configuration
config_out = open (FullIniFile, 'w')
config.write ( config_out )
config_out.close ()

if ICO :
    if SortIco : 
        # Creation d'une clef de tri pour les restarts (pour chaque restart !)
        DYN_beg_keysort = np.lexsort ( (d_DYN_beg['lat_mesh'], d_DYN_beg['lon_mesh'] ) )
        ATM_beg_keysort = np.lexsort ( (d_ATM_beg['latitude'], d_ATM_beg['longitude']) )
        SRF_beg_keysort = np.lexsort ( (d_SRF_beg['nav_lat' ], d_SRF_beg['nav_lon']  ) )
        DYN_end_keysort = np.lexsort ( (d_DYN_end['lat_mesh'], d_DYN_end['lon_mesh'] ) )
        ATM_end_keysort = np.lexsort ( (d_ATM_end['latitude'], d_ATM_end['longitude']) )
        SRF_end_keysort = np.lexsort ( (d_SRF_end['nav_lat' ], d_SRF_end['nav_lon']  ) )
        
        if ATM_HIS == 'ico' :
            ATM_his_keysort = np.lexsort ( (d_ATM_his['lat'], d_ATM_his['lon'] ) )
        if SRF_HIS == 'ico' :
            SRF_his_keysort = np.lexsort ( (d_SRF_his['lat'], d_SRF_his['lon'] ) )
            RUN_his_keysort = SRF_his_keysort
    else : 
        DYN_beg_keysort = np.arange ( len ( d_DYN_beg['lat_mesh'] ) )
        ATM_beg_keysort = DYN_beg_keysort
        SRF_beg_keysort = DYN_beg_keysort

        DYN_end_keysort = DYN_beg_keysort
        ATM_end_keysort = DYN_beg_keysort
        SRF_end_keysort = DYN_beg_keysort
    
        ATM_his_keysort = DYN_beg_keysort
        SRF_his_keysort = DYN_beg_keysort
        RUN_his_keysort = SRF_his_keysort
        
# ATM grid with cell surfaces
if LMDZ :
    echo ('ATM grid with cell surfaces : LMDZ')
    ATM_lat       = lmdz.geo2point (   rprec (d_ATM_his ['lat'])+0*rprec (d_ATM_his ['lon']), dim1D='cell' )
    ATM_lon       = lmdz.geo2point ( 0*rprec (d_ATM_his ['lat'])+  rprec (d_ATM_his ['lon']), dim1D='cell' )
    ATM_aire      = lmdz.geo2point ( rprec (d_ATM_his ['aire'] )    [0], cumulPoles=True, dim1D='cell' )
    ATM_fter      = lmdz.geo2point ( rprec (d_ATM_his ['fract_ter'][0]), dim1D='cell' )
    ATM_foce      = lmdz.geo2point ( rprec (d_ATM_his ['fract_oce'][0]), dim1D='cell' )
    ATM_fsic      = lmdz.geo2point ( rprec (d_ATM_his ['fract_sic'][0]), dim1D='cell' )
    ATM_flic      = lmdz.geo2point ( rprec (d_ATM_his ['fract_lic'][0]), dim1D='cell' )
    SRF_lat       = lmdz.geo2point (   rprec (d_SRF_his ['lat'])+0*rprec (d_SRF_his ['lon']), dim1D='cell' )
    SRF_lon       = lmdz.geo2point ( 0*rprec (d_SRF_his ['lat'])+  rprec (d_SRF_his ['lon']), dim1D='cell' )
    SRF_aire      = lmdz.geo2point ( rprec (d_SRF_his ['Areas']) * rprec (d_SRF_his ['Contfrac']), dim1D='cell', cumulPoles=True )
    SRF_areas     = lmdz.geo2point ( rprec (d_SRF_his ['Areas'])  ,  dim1D='cell', cumulPoles=True )
    SRF_contfrac  = lmdz.geo2point ( rprec (d_SRF_his ['Contfrac']), dim1D='cell' )

if ICO :
    if ATM_HIS == 'latlon' :
        echo ( 'ATM areas and fractions on latlon grid' )
        if 'lat_dom_out' in d_ATM_his.variables :
            ATM_lat  = lmdz.geo2point (   rprec (d_ATM_his ['lat_dom_out'])+0*rprec (d_ATM_his ['lon_dom_out']), dim1D='cell' )
            ATM_lon  = lmdz.geo2point ( 0*rprec (d_ATM_his ['lat_dom_out'])+  rprec (d_ATM_his ['lon_dom_out']), dim1D='cell' )
        else :
            ATM_lat  = lmdz.geo2point (   rprec (d_ATM_his ['lat'])+0*rprec (d_ATM_his ['lon']), dim1D='cell' )
            ATM_lon  = lmdz.geo2point ( 0*rprec (d_ATM_his ['lat'])+  rprec (d_ATM_his ['lon']), dim1D='cell' )
        ATM_aire = lmdz.geo2point ( rprec (d_ATM_his ['aire'][0]).squeeze(), cumulPoles=True, dim1D='cell' )
        ATM_fter = lmdz.geo2point ( rprec (d_ATM_his ['fract_ter'][0]), dim1D='cell' )
        ATM_foce = lmdz.geo2point ( rprec (d_ATM_his ['fract_oce'][0]), dim1D='cell' )
        ATM_fsic = lmdz.geo2point ( rprec (d_ATM_his ['fract_sic'][0]), dim1D='cell' )
        ATM_flic = lmdz.geo2point ( rprec (d_ATM_his ['fract_lic'][0]), dim1D='cell' )

    if ATM_HIS == 'ico' :
        echo ( 'ATM areas and fractions on ICO grid' )
        ATM_aire =  rprec (d_ATM_his ['aire']     [0])[ATM_his_keysort].squeeze()
        ATM_lat  =  rprec (d_ATM_his ['lat']         )[ATM_his_keysort]
        ATM_lon  =  rprec (d_ATM_his ['lon']         )[ATM_his_keysort]
        ATM_fter =  rprec (d_ATM_his ['fract_ter'][0])[ATM_his_keysort]
        ATM_foce =  rprec (d_ATM_his ['fract_oce'][0])[ATM_his_keysort]
        ATM_fsic =  rprec (d_ATM_his ['fract_sic'][0])[ATM_his_keysort]
        ATM_flic =  rprec (d_ATM_his ['fract_lic'][0])[ATM_his_keysort]

    if SRF_HIS == 'latlon' :
        echo ( 'SRF areas and fractions on latlon grid' )
        if 'lat_domain_landpoints_out' in d_SRF_his  : 
            SRF_lat  = lmdz.geo2point (   rprec (d_SRF_his ['lat_domain_landpoints_out'])+0*rprec (d_SRF_his ['lon_domain_landpoints_out']), dim1D='cell' )
            SRF_lon  = lmdz.geo2point ( 0*rprec (d_SRF_his ['lat_domain_landpoints_out'])+  rprec (d_SRF_his ['lon_domain_landpoints_out']), dim1D='cell' )
        else : 
            if 'lat_domain_landpoints_out' in d_SRF_his :
                SRF_lat  = lmdz.geo2point (   rprec (d_SRF_his ['lat_dom_out'])+0*rprec (d_SRF_his ['lon_dom_out']), dim1D='cell' )
                SRF_lon  = lmdz.geo2point ( 0*rprec (d_SRF_his ['lat_dom_out'])+  rprec (d_SRF_his ['lon_dom_out']), dim1D='cell' )
            else :
                SRF_lat  = lmdz.geo2point (   rprec (d_SRF_his ['lat'])+0*rprec (d_SRF_his ['lon']), dim1D='cell' )
                SRF_lon  = lmdz.geo2point ( 0*rprec (d_SRF_his ['lat'])+  rprec (d_SRF_his ['lon']), dim1D='cell' )
                
        SRF_areas     = lmdz.geo2point ( rprec (d_SRF_his ['Areas']   )   , dim1D='cell', cumulPoles=True )
        SRF_areafrac  = lmdz.geo2point ( rprec (d_SRF_his ['AreaFrac'])   , dim1D='cell', cumulPoles=True )
        SRF_contfrac  = lmdz.geo2point ( rprec (d_SRF_his ['Contfrac'])   , dim1D='cell', cumulPoles=True )
        SRF_aire = SRF_areafrac
 
    if SRF_HIS == 'ico' :
        echo ( 'SRF areas and fractions on latlon grid' )
        SRF_lat       =  rprec (d_SRF_his ['lat']     ) [SRF_his_keysort]
        SRF_lon       =  rprec (d_SRF_his ['lon']     ) [SRF_his_keysort]
        SRF_areas     =  rprec (d_SRF_his ['Areas']   ) [SRF_his_keysort] 
        SRF_contfrac  =  rprec (d_SRF_his ['Contfrac']) [SRF_his_keysort]
        SRF_aire      =  SRF_areas * SRF_contfrac

ATM_fsea      = ATM_foce + ATM_fsic
ATM_flnd      = ATM_fter + ATM_flic
ATM_aire_fter = ATM_aire * ATM_fter
ATM_aire_flic = ATM_aire * ATM_flic
ATM_aire_fsic = ATM_aire * ATM_fsic
ATM_aire_foce = ATM_aire * ATM_foce
ATM_aire_flnd = ATM_aire * ATM_flnd
ATM_aire_fsea = ATM_aire * ATM_fsea

#SRF_aire = SRF_aire.where ( np.abs (SRF_aire) < 1E15, 0. )

## Write the full configuration
config_out = open (FullIniFile, 'w')
config.write ( config_out )
config_out.close ()
            
if ICO :
    # Area on icosahedron grid
    d_DYN_aire = xr.open_dataset ( file_DYN_aire, decode_times=False ).squeeze()

    if SortIco : 
        # Creation d'une clef de tri pour le fichier aire
        DYN_aire_keysort = np.lexsort ( (d_DYN_aire['lat'], d_DYN_aire['lon']) )
    else : 
        DYN_aire_keysort = np.arange ( len ( d_DYN_aire['lat'] ) )
            
    DYN_lat = d_DYN_aire['lat'][DYN_aire_keysort]
    DYN_lon = d_DYN_aire['lon'][DYN_aire_keysort]

    DYN_aire = d_DYN_aire['aire'][DYN_aire_keysort]
    DYN_fsea = d_DYN_aire['fract_oce'][DYN_aire_keysort] + d_DYN_aire['fract_sic'][DYN_aire_keysort]
    DYN_flnd = 1.0 - DYN_fsea
    DYN_fter = d_ATM_beg['FTER'][ATM_beg_keysort]
    DYN_flic = d_ATM_beg['FLIC'][ATM_beg_keysort]
    DYN_aire_fter = DYN_aire * DYN_fter
    
if LMDZ :
    # Area on lon/lat grid
    DYN_aire = ATM_aire
    DYN_fsea = ATM_fsea
    DYN_flnd = ATM_flnd
    DYN_fter = rprec (d_ATM_beg['FTER'])
    DYN_flic = rprec (d_ATM_beg['FLIC'])
    DYN_aire_fter = DYN_aire * DYN_fter

# Functions computing integrals and sum
def DYN_stock_int (stock) :
    '''Integrate (* surface) stock on atmosphere grid'''
    DYN_stock_int  = wu.Psum ( (stock * DYN_aire).to_masked_array().ravel() ) 
    return DYN_stock_int

def ATM_flux_int (flux) :
    '''Integrate (* time * surface) flux on atmosphere grid'''
    ATM_flux_int  = wu.Psum ( (flux * dtime_per_sec * ATM_aire).to_masked_array().ravel() )
    return ATM_flux_int

def LIC_flux_int (flux) :
    '''Integrate (* time * surface) flux on land ice grid'''
    LIC_flux_int  = wu.Psum ( (flux * dtime_per_sec * ATM_aire_flic).to_masked_array().ravel() )
    return LIC_flux_int

def SRF_stock_int (stock) :
    '''Integrate (* surface) stock on land grid'''
    #SRF_stock_int  = wu.Ksum (  ( (stock * DYN_aire_fter).to_masked_array().ravel()) ) # Marche avec ICO et lon/lat na
    SRF_stock_int  = wu.Ksum (  ( (stock * DYN_aire_fter).to_masked_array().ravel()) )
    return SRF_stock_int

def SRF_flux_int (flux) :
    '''Integrate (* time * surface) flux on land grid'''
    SRF_flux_int  = wu.Psum (  (flux * dtime_per_sec * SRF_aire).to_masked_array().ravel() )
    return SRF_flux_int

def ONE_stock_int (stock) :
    '''Sum stock '''
    ONE_stock_int  = wu.Psum ( stock.to_masked_array().ravel() )
    return ONE_stock_int

def ONE_flux_int (flux) :
    '''Integrate (* time) flux on area=1 grid '''
    ONE_flux_int  = wu.Psum ( (flux * dtime_per_sec ).to_masked_array().ravel() )
    return ONE_flux_int

def Sprec ( tlist ) :
    '''Accurate sum of list of scalar elements'''
    return wu.Psum ( np.array ( tlist) )

ATM_aire_sea     = ATM_aire * ATM_fsea

ATM_aire_tot     = ONE_stock_int (ATM_aire)
ATM_aire_sea_tot = ONE_stock_int (ATM_aire_fsea)
ATM_aire_ter_tot = ONE_stock_int (ATM_aire_fter)
ATM_aire_lic_tot = ONE_stock_int (ATM_aire_flic)

DYN_aire_tot     = ONE_stock_int ( DYN_aire )
SRF_aire_tot     = ONE_stock_int ( SRF_aire )

echo ('')
echo ( f'ATM DYN : Area of atmosphere        : {DYN_aire_tot:18.9e}' )
echo ( f'ATM HIS : Area of atmosphere        : {ATM_aire_tot:18.9e}' )
echo ( f'ATM HIS : Area of ter in atmosphere : {ATM_aire_ter_tot:18.9e}' )
echo ( f'ATM HIS : Area of lic in atmosphere : {ATM_aire_lic_tot:18.9e}' )
echo ( f'ATM SRF : Area of atmosphere        : {SRF_aire_tot:18.9e}' )
echo ('')
echo ( 'ATM DYN : Area of atmosphere/(4pi R^2)        : {:18.9f}'.format(DYN_aire_tot/(Ra*Ra*4*np.pi) ) )
echo ( 'ATM HIS : Area of atmosphere/(4pi R^2)        : {:18.9f}'.format(ATM_aire_tot/(Ra*Ra*4*np.pi) ) )
echo ( 'ATM HIS : Area of ter in atmosphere/(4pi R^2) : {:18.9f}'.format(ATM_aire_ter_tot/(Ra*Ra*4*np.pi) ) )
echo ( 'ATM SRF : Area of atmosphere/(4pi R^2)        : {:18.9f}'.format(SRF_aire_tot/(Ra*Ra*4*np.pi) ) )
echo ('')
echo ( f'ATM SRF : Area of atmosphere (no contfrac): {ONE_stock_int (SRF_areas):18.9e}' )


if (  np.abs (ATM_aire_tot/(Ra*Ra*4*np.pi) - 1.0) > 0.01 ) :
    raise Exception ('Error of atmosphere surface interpolated on lon/lat grid')

echo ( '\n====================================================================================' )
echo ( f'-- ATM changes in stores  -- {Title} ' )

#-- Change in precipitable water from the atmosphere daily and monthly files
#-- Compute sum weighted by gridcell area (kg/m2) then convert to Sv

echo ( 'ATM vertical grid' )
ATM_Ahyb = d_ATM_his['Ahyb'].squeeze()
ATM_Bhyb = d_ATM_his['Bhyb'].squeeze()

echo ( 'Surface pressure' )
if ICO :
    DYN_psol_beg = d_DYN_beg['ps'][DYN_beg_keysort]
    DYN_psol_end = d_DYN_end['ps'][DYN_end_keysort]
if LMDZ : 
    DYN_psol_beg = lmdz.geo2point ( d_DYN_beg['ps'].isel(rlonv=slice(0,-1)), dim1D='cell' )
    DYN_psol_end = lmdz.geo2point ( d_DYN_end['ps'].isel(rlonv=slice(0,-1)), dim1D='cell' )
    
echo ( '3D Pressure at the interface layers (not scalar points)' )
DYN_pres_beg = ATM_Ahyb + ATM_Bhyb * DYN_psol_beg
DYN_pres_end = ATM_Ahyb + ATM_Bhyb * DYN_psol_end

echo ( 'Check computation of pressure levels' )

ind = np.empty (8)

ind[0] = (DYN_pres_beg[ 0]-DYN_psol_beg).min().item()
ind[1] = (DYN_pres_beg[ 0]-DYN_psol_beg).max().item()
ind[2] = (DYN_pres_beg[-1]).min().item()
ind[3] = (DYN_pres_beg[-1]).max().item()
ind[4] = (DYN_pres_end[ 0]-DYN_psol_end).min().item()
ind[5] = (DYN_pres_end[ 0]-DYN_psol_end).max().item()
ind[6] = (DYN_pres_end[-1]).min().item()
ind[7] = (DYN_pres_end[-1]).max().item()

if any ( ind != 0) :
    echo ( 'All values should be zero' )
    echo ( f'(DYN_pres_beg[ 0]-DYN_psol_beg).min().item() = {ind[0]}' )
    echo ( f'(DYN_pres_beg[ 0]-DYN_psol_beg).max().item() = {ind[1]}' )
    echo ( f'(DYN_pres_beg[-1]).min().item()              = {ind[2]}' )
    echo ( f'(DYN_pres_beg[-1]).max().item()              = {ind[3]}' )
    echo ( f'(DYN_pres_end[ 0]-DYN_psol_end).min().item() = {ind[4]}' )
    echo ( f'(DYN_pres_end[ 0]-DYN_psol_end).max().item() = {ind[5]}' )
    echo ( f'(DYN_pres_end[-1]).min().item()              = {ind[6]}' )
    echo ( f'(DYN_pres_end[-1]).max().item()              = {ind[7]}' )
    raise Exception
    
klevp1 = ATM_Bhyb.shape[-1]
cell   = DYN_psol_beg.shape[-1]
klev   = klevp1 - 1

echo ( 'Layer thickness (pressure)' )
DYN_mass_beg = xr.DataArray ( np.empty( (klev, cell)), dims = ('sigs', 'cell'), coords=(np.arange(klev), np.arange(cell) ) )
DYN_mass_end = xr.DataArray ( np.empty( (klev, cell)), dims = ('sigs', 'cell'), coords=(np.arange(klev), np.arange(cell) ) )
    
for k in np.arange (klev) :
    DYN_mass_beg[k,:] = ( DYN_pres_beg[k,:] - DYN_pres_beg[k+1,:] ) / Grav
    DYN_mass_end[k,:] = ( DYN_pres_end[k,:] - DYN_pres_end[k+1,:] ) / Grav

DYN_mass_beg_2D = DYN_mass_beg.sum (dim='sigs')
DYN_mass_end_2D = DYN_mass_end.sum (dim='sigs')

DYN_mas_air_beg = DYN_stock_int ( DYN_mass_beg_2D )
DYN_mas_air_end = DYN_stock_int ( DYN_mass_end_2D )

echo ( 'Vertical and horizontal integral, and sum of liquid, solid and vapor water phases' )
if LMDZ :
    if 'H2Ov' in d_DYN_beg.variables :
        echo ('reading LATLON : H2Ov, H2Ol, H2Oi' )
        DYN_wat_beg = lmdz.geo3point ( d_DYN_beg['H2Ov'] + d_DYN_beg['H2Ol'] + d_DYN_beg['H2Oi'].isel(rlonv=slice(0,-1) ), dim1D='cell' )
        DYN_wat_end = lmdz.geo3point ( d_DYN_end['H2Ov'] + d_DYN_end['H2Ol'] + d_DYN_end['H2Oi'].isel(rlonv=slice(0,-1) ), dim1D='cell' )
    if 'H2Ov_g' in d_DYN_beg.variables :
        echo ('reading LATLON : H2O_g, H2O_l, H2O_s' )
        DYN_wat_beg = lmdz.geo3point ( (d_DYN_beg['H2O_g'] + d_DYN_beg['H2O_l'] + d_DYN_beg['H2O_s']).isel(rlonv=slice(0,-1) ), dim1D='cell' )
        DYN_wat_end = lmdz.geo3point ( (d_DYN_end['H2O_g'] + d_DYN_end['H2O_l'] + d_DYN_end['H2O_s']).isel(rlonv=slice(0,-1) ), dim1D='cell' )
if ICO :
    if 'H2Ov_g' in d_DYN_beg.variables :
        echo ('reading ICO : H2O_g, H2O_l, H2O_s' )
        DYN_wat_beg = (d_DYN_beg['H2O_g'] + d_DYN_beg['H2O_l'] + d_DYN_beg['H2O_s'])[..., DYN_beg_keysort]
        DYN_wat_end = (d_DYN_end['H2O_g'] + d_DYN_end['H2O_l'] + d_DYN_end['H2O_s'])[..., DYN_beg_keysort]
    elif 'H2O_g' in d_DYN_beg.variables :
        echo ('reading ICO : H2O_g, H2O_l, H2O_s' )
        DYN_wat_beg = (d_DYN_beg['H2O_g'] + d_DYN_beg['H2O_l'] + d_DYN_beg['H2O_s'])[..., DYN_beg_keysort]
        DYN_wat_end = (d_DYN_end['H2O_g'] + d_DYN_end['H2O_l'] + d_DYN_end['H2O_s'])[..., DYN_beg_keysort]
    elif 'q' in d_DYN_beg.variables :
        echo ('reading ICO : q' )
        DYN_wat_beg = (d_DYN_beg['q'].isel(nq=0) + d_DYN_beg['q'].isel(nq=1) + d_DYN_beg['q'].isel(nq=2) )[..., DYN_beg_keysort]
        DYN_wat_end = (d_DYN_end['q'].isel(nq=0) + d_DYN_end['q'].isel(nq=1) + d_DYN_end['q'].isel(nq=2) )[..., DYN_beg_keysort]

if 'lev' in DYN_wat_beg.dims : 
    DYN_wat_beg = DYN_wat_beg.rename ( {'lev':'sigs'} )
    DYN_wat_end = DYN_wat_end.rename ( {'lev':'sigs'} )
    
echo ( 'Compute water content : vertical and horizontal integral' )

DYN_wat_beg_2D =  (DYN_mass_beg * DYN_wat_beg).sum (dim='sigs')
DYN_wat_end_2D =  (DYN_mass_end * DYN_wat_end).sum (dim='sigs')

DYN_mas_wat_beg = DYN_stock_int ( DYN_wat_beg_2D )
DYN_mas_wat_end = DYN_stock_int ( DYN_wat_end_2D )

echo ( 'Variation of water content' )
dDYN_mas_wat = DYN_mas_wat_end - DYN_mas_wat_beg

echo ( f'\nChange of atmosphere water content (dynamics)  -- {Title} ' )
echo ( '------------------------------------------------------------------------------------' )
echo ( 'DYN_mas_air_beg = {:12.6e} kg | DYN_mas_air_end = {:12.6e} kg'.format (DYN_mas_air_beg, DYN_mas_air_end) )
echo ( 'DYN_mas_wat_beg = {:12.6e} kg | DYN_mas_wat_end = {:12.6e} kg'.format (DYN_mas_wat_beg, DYN_mas_wat_end) )
prtFlux ( 'dMass (atm)  ', dDYN_mas_wat, 'e', True )

ATM_sno_beg = d_ATM_beg['SNOW01']*d_ATM_beg['FTER'] + d_ATM_beg['SNOW02']*d_ATM_beg['FLIC'] + \
              d_ATM_beg['SNOW03']*d_ATM_beg['FOCE'] + d_ATM_beg['SNOW04']*d_ATM_beg['FSIC']
ATM_sno_end = d_ATM_end['SNOW01']*d_ATM_end['FTER'] + d_ATM_end['SNOW02']*d_ATM_end['FLIC'] + \
              d_ATM_end['SNOW03']*d_ATM_end['FOCE'] + d_ATM_end['SNOW04']*d_ATM_end['FSIC']

ATM_qs_beg  = d_ATM_beg['QS01']  *d_ATM_beg['FTER'] + d_ATM_beg['QS02']  *d_ATM_beg['FLIC'] + \
              d_ATM_beg['QS03']  *d_ATM_beg['FOCE'] + d_ATM_beg['QS04']  *d_ATM_beg['FSIC']
ATM_qs_end  = d_ATM_end['QS01']  *d_ATM_end['FTER'] + d_ATM_end['QS02']  *d_ATM_end['FLIC'] + \
              d_ATM_end['QS03']  *d_ATM_end['FOCE'] + d_ATM_end['QS04']  *d_ATM_end['FSIC']

ATM_qsol_beg     = d_ATM_beg['QSOL']
ATM_qsol_end     = d_ATM_end['QSOL']

LIC_sno_beg      = d_ATM_beg['SNOW02']*d_ATM_beg['FLIC']
LIC_sno_end      = d_ATM_end['SNOW02']*d_ATM_end['FLIC']

LIC_runlic0_beg  = d_ATM_beg['RUNOFFLIC0']
LIC_runlic0_end  = d_ATM_end['RUNOFFLIC0']

ATM_qs01_beg     = d_ATM_beg['QS01'] * d_ATM_beg['FTER']
ATM_qs02_beg     = d_ATM_beg['QS02'] * d_ATM_beg['FLIC']
ATM_qs03_beg     = d_ATM_beg['QS03'] * d_ATM_beg['FOCE']
ATM_qs04_beg     = d_ATM_beg['QS04'] * d_ATM_beg['FSIC']

ATM_qs01_end     = d_ATM_end['QS01'] * d_ATM_end['FTER']
ATM_qs02_end     = d_ATM_end['QS02'] * d_ATM_end['FLIC']
ATM_qs03_end     = d_ATM_end['QS03'] * d_ATM_end['FOCE']
ATM_qs04_end     = d_ATM_end['QS04'] * d_ATM_end['FSIC']  

if ICO :
     ATM_sno_beg     = ATM_sno_beg    [ATM_beg_keysort]
     ATM_sno_end     = ATM_sno_end    [ATM_end_keysort]
     ATM_qs_beg      = ATM_qs_beg     [ATM_beg_keysort]
     ATM_qs_end      = ATM_qs_end     [ATM_end_keysort]
     ATM_qsol_beg    = ATM_qsol_beg   [ATM_beg_keysort]
     ATM_qs01_beg    = ATM_qs01_beg   [ATM_beg_keysort]
     ATM_qs02_beg    = ATM_qs02_beg   [ATM_beg_keysort]
     ATM_qs03_beg    = ATM_qs03_beg   [ATM_beg_keysort]
     ATM_qs04_beg    = ATM_qs04_beg   [ATM_beg_keysort]
     ATM_qsol_end    = ATM_qsol_end   [ATM_end_keysort]
     ATM_qs01_end    = ATM_qs01_end   [ATM_end_keysort]
     ATM_qs02_end    = ATM_qs02_end   [ATM_end_keysort]
     ATM_qs03_end    = ATM_qs03_end   [ATM_end_keysort]
     ATM_qs04_end    = ATM_qs04_end   [ATM_end_keysort]
     LIC_sno_beg     = LIC_sno_beg    [ATM_beg_keysort]
     LIC_sno_end     = LIC_sno_end    [ATM_end_keysort]
     LIC_runlic0_beg = LIC_runlic0_beg[ATM_beg_keysort]
     LIC_runlic0_end = LIC_runlic0_end[ATM_end_keysort]
   
LIC_qs_beg = ATM_qs02_beg
LIC_qs_end = ATM_qs02_end

ATM_mas_sno_beg     = DYN_stock_int ( ATM_sno_beg  )
ATM_mas_sno_end     = DYN_stock_int ( ATM_sno_end  )

ATM_mas_qs_beg      = DYN_stock_int ( ATM_qs_beg   )
ATM_mas_qs_end      = DYN_stock_int ( ATM_qs_end   )
ATM_mas_qsol_beg    = DYN_stock_int ( ATM_qsol_beg )
ATM_mas_qs01_beg    = DYN_stock_int ( ATM_qs01_beg )
ATM_mas_qs02_beg    = DYN_stock_int ( ATM_qs02_beg )
ATM_mas_qs03_beg    = DYN_stock_int ( ATM_qs03_beg )
ATM_mas_qs04_beg    = DYN_stock_int ( ATM_qs04_beg )
ATM_mas_qsol_end    = DYN_stock_int ( ATM_qsol_end )
ATM_mas_qs01_end    = DYN_stock_int ( ATM_qs01_end )
ATM_mas_qs02_end    = DYN_stock_int ( ATM_qs02_end )
ATM_mas_qs03_end    = DYN_stock_int ( ATM_qs03_end )
ATM_mas_qs04_end    = DYN_stock_int ( ATM_qs04_end )

LIC_mas_sno_beg     = DYN_stock_int ( LIC_sno_beg     )
LIC_mas_sno_end     = DYN_stock_int ( LIC_sno_end     )
LIC_mas_runlic0_beg = DYN_stock_int ( LIC_runlic0_beg )
LIC_mas_runlic0_end = DYN_stock_int ( LIC_runlic0_end )

LIC_mas_qs_beg  = ATM_mas_qs02_beg
LIC_mas_qs_end  = ATM_mas_qs02_end
LIC_mas_wat_beg = LIC_mas_qs_beg + LIC_mas_sno_beg
LIC_mas_wat_end = LIC_mas_qs_end + LIC_mas_sno_end

dATM_mas_sno  = ATM_mas_sno_end  - ATM_mas_sno_beg
dATM_mas_qs   = ATM_mas_qs_end   - ATM_mas_qs_beg
dATM_mas_qsol = ATM_mas_qsol_end - ATM_mas_qsol_beg

dATM_mas_qs01 = ATM_mas_qs01_end - ATM_mas_qs01_beg
dATM_mas_qs02 = ATM_mas_qs02_end - ATM_mas_qs02_beg
dATM_mas_qs03 = ATM_mas_qs03_end - ATM_mas_qs03_beg
dATM_mas_qs04 = ATM_mas_qs04_end - ATM_mas_qs04_beg

dLIC_mas_qs      = LIC_mas_qs_end      - LIC_mas_qs_beg
dLIC_mas_sno     = LIC_mas_sno_end     - LIC_mas_sno_beg
dLIC_mas_runlic0 = LIC_mas_runlic0_end - LIC_mas_runlic0_beg

dLIC_mas_wat     = dLIC_mas_qs + dLIC_mas_sno # + dLIC_mas_runlic0

echo ( f'\nChange of atmosphere snow content (Land ice) -- {Title} ' )
echo ( '------------------------------------------------------------------------------------' )
echo ( 'ATM_mas_sno_beg  = {:12.6e} kg | ATM_mas_sno_end  = {:12.6e} kg'.format (ATM_mas_sno_beg, ATM_mas_sno_end) )
prtFlux ( 'dMass (neige atm) ', dATM_mas_sno  , 'e', True  )

echo ( f'\nChange of soil humidity content -- {Title} ' )
echo ( '------------------------------------------------------------------------------------' )
echo ( 'ATM_mas_qs_beg  = {:12.6e} kg | ATM_mas_qs_end  = {:12.6e} kg'.format (ATM_mas_qs_beg, ATM_mas_qs_end) )
prtFlux ( 'dMass (neige atm) ', dATM_mas_qs, 'e', True )

echo ( f'\nChange of atmosphere water+snow content -- {Title}  ' )
echo ( '------------------------------------------------------------------------------------' )
prtFlux ( 'dMass (eau + neige atm) ', dDYN_mas_wat + dATM_mas_sno , 'e', True)

echo ( '\n====================================================================================' )
echo ( f'-- SRF changes  -- {Title} ' )

if Routing == 'SIMPLE' :
    RUN_mas_wat_fast_beg   = ONE_stock_int ( d_RUN_beg ['fast_reservoir']   )
    RUN_mas_wat_slow_beg   = ONE_stock_int ( d_RUN_beg ['slow_reservoir']   )
    RUN_mas_wat_stream_beg = ONE_stock_int ( d_RUN_beg ['stream_reservoir'] )
    
    RUN_mas_wat_flood_beg  = ONE_stock_int ( d_SRF_beg ['floodres']  )
    RUN_mas_wat_lake_beg   = ONE_stock_int ( d_SRF_beg ['lakeres']   )
    RUN_mas_wat_pond_beg   = ONE_stock_int ( d_SRF_beg ['pondres']   )
    
    RUN_mas_wat_fast_end   = ONE_stock_int ( d_RUN_end ['fast_reservoir']   )
    RUN_mas_wat_slow_end   = ONE_stock_int ( d_RUN_end ['slow_reservoir']   )
    RUN_mas_wat_stream_end = ONE_stock_int ( d_RUN_end ['stream_reservoir'] )
    
    RUN_mas_wat_flood_end  = ONE_stock_int ( d_SRF_end ['floodres']  )
    RUN_mas_wat_lake_end   = ONE_stock_int ( d_SRF_end ['lakeres']   )
    RUN_mas_wat_pond_end   = ONE_stock_int ( d_SRF_end ['pondres']   )

if Routing == 'SECHIBA' :
    RUN_mas_wat_fast_beg   = ONE_stock_int ( d_SRF_beg ['fastres']   )
    RUN_mas_wat_slow_beg   = ONE_stock_int ( d_SRF_beg ['slowres']   )
    RUN_mas_wat_stream_beg = ONE_stock_int ( d_SRF_beg ['streamres'] )
    RUN_mas_wat_flood_beg  = ONE_stock_int ( d_SRF_beg ['floodres']  )
    RUN_mas_wat_lake_beg   = ONE_stock_int ( d_SRF_beg ['lakeres']   )
    RUN_mas_wat_pond_beg   = ONE_stock_int ( d_SRF_beg ['pondres']   )
    
    RUN_mas_wat_fast_end   = ONE_stock_int ( d_SRF_end ['fastres']   )
    RUN_mas_wat_slow_end   = ONE_stock_int ( d_SRF_end ['slowres']   )
    RUN_mas_wat_stream_end = ONE_stock_int ( d_SRF_end ['streamres'] )
    RUN_mas_wat_flood_end  = ONE_stock_int ( d_SRF_end ['floodres']  )
    RUN_mas_wat_lake_end   = ONE_stock_int ( d_SRF_end ['lakeres']   )
    RUN_mas_wat_pond_end   = ONE_stock_int ( d_SRF_end ['pondres']   )

RUN_mas_wat_beg = Sprec ( [RUN_mas_wat_fast_beg , RUN_mas_wat_slow_beg, RUN_mas_wat_stream_beg,
                           RUN_mas_wat_flood_beg, RUN_mas_wat_lake_beg, RUN_mas_wat_pond_beg] )
                  
RUN_mas_wat_end = Sprec ( [RUN_mas_wat_fast_end  , RUN_mas_wat_slow_end , RUN_mas_wat_stream_end,
                           RUN_mas_wat_flood_end , RUN_mas_wat_lake_end , RUN_mas_wat_pond_end] )

dRUN_mas_wat_fast   = RUN_mas_wat_fast_end   - RUN_mas_wat_fast_beg
dRUN_mas_wat_slow   = RUN_mas_wat_slow_end   - RUN_mas_wat_slow_beg
dRUN_mas_wat_stream = RUN_mas_wat_stream_end - RUN_mas_wat_stream_beg
dRUN_mas_wat_flood  = RUN_mas_wat_flood_end  - RUN_mas_wat_flood_beg
dRUN_mas_wat_lake   = RUN_mas_wat_lake_end   - RUN_mas_wat_lake_beg
dRUN_mas_wat_pond   = RUN_mas_wat_pond_end   - RUN_mas_wat_pond_beg

dRUN_mas_wat        = RUN_mas_wat_end        - RUN_mas_wat_beg

echo ( f'\nRunoff reservoirs -- {Title} ')
echo ( f'------------------------------------------------------------------------------------' )
echo ( f'RUN_mas_wat_fast_beg   = {RUN_mas_wat_fast_beg  :12.6e} kg | RUN_mas_wat_fast_end   = {RUN_mas_wat_fast_end  :12.6e} kg ' )
echo ( f'RUN_mas_wat_slow_beg   = {RUN_mas_wat_slow_beg  :12.6e} kg | RUN_mas_wat_slow_end   = {RUN_mas_wat_slow_end  :12.6e} kg ' )
echo ( f'RUN_mas_wat_stream_beg = {RUN_mas_wat_stream_beg:12.6e} kg | RUN_mas_wat_stream_end = {RUN_mas_wat_stream_end:12.6e} kg ' )
echo ( f'RUN_mas_wat_flood_beg  = {RUN_mas_wat_flood_beg :12.6e} kg | RUN_mas_wat_flood_end  = {RUN_mas_wat_flood_end :12.6e} kg ' )
echo ( f'RUN_mas_wat_lake_beg   = {RUN_mas_wat_lake_beg  :12.6e} kg | RUN_mas_wat_lake_end   = {RUN_mas_wat_lake_end  :12.6e} kg ' )
echo ( f'RUN_mas_wat_pond_beg   = {RUN_mas_wat_pond_beg  :12.6e} kg | RUN_mas_wat_pond_end   = {RUN_mas_wat_pond_end  :12.6e} kg ' )
echo ( f'RUN_mas_wat_beg        = {RUN_mas_wat_beg       :12.6e} kg | RUN_mas_wat_end        = {RUN_mas_wat_end       :12.6e} kg ' )

echo ( '------------------------------------------------------------------------------------' )

prtFlux ( 'dMass (fast)   ', dRUN_mas_wat_fast  , 'e', True )
prtFlux ( 'dMass (slow)   ', dRUN_mas_wat_slow  , 'e', True )
prtFlux ( 'dMass (stream) ', dRUN_mas_wat_stream, 'e', True )
prtFlux ( 'dMass (flood)  ', dRUN_mas_wat_flood , 'e', True )
prtFlux ( 'dMass (lake)   ', dRUN_mas_wat_lake  , 'e', True )
prtFlux ( 'dMass (pond)   ', dRUN_mas_wat_pond  , 'e', True )
prtFlux ( 'dMass (all)    ', dRUN_mas_wat       , 'e', True )

echo ( f'\nWater content in routing  -- {Title} ' )
echo ( '------------------------------------------------------------------------------------' )
echo ( f'RUN_mas_wat_beg = {RUN_mas_wat_end:12.6e} kg | RUN_mas_wat_end = {RUN_mas_wat_end:12.6e} kg' )
prtFlux ( 'dMass (routing) ', dRUN_mas_wat , 'e', True   )

echo ( '\n====================================================================================' )
print (f'Reading SRF restart')
SRF_tot_watveg_beg  = d_SRF_beg['tot_watveg_beg']  ; SRF_tot_watveg_beg  = SRF_tot_watveg_beg .where (SRF_tot_watveg_beg  < 1E15, 0.)
SRF_tot_watsoil_beg = d_SRF_beg['tot_watsoil_beg'] ; SRF_tot_watsoil_beg = SRF_tot_watsoil_beg.where (SRF_tot_watsoil_beg < 1E15, 0.)
SRF_snow_beg        = d_SRF_beg['snow_beg']        ; SRF_snow_beg        = SRF_snow_beg       .where (SRF_snow_beg        < 1E15, 0.)
SRF_lakeres_beg     = d_SRF_beg['lakeres']         ; SRF_lakeres_beg     = SRF_lakeres_beg    .where (SRF_lakeres_beg     < 1E15, 0.)

SRF_tot_watveg_end  = d_SRF_end['tot_watveg_beg']  ; SRF_tot_watveg_end  = SRF_tot_watveg_end .where (SRF_tot_watveg_end  < 1E15, 0.)
SRF_tot_watsoil_end = d_SRF_end['tot_watsoil_beg'] ; SRF_tot_watsoil_end = SRF_tot_watsoil_end.where (SRF_tot_watsoil_end < 1E15, 0.)
SRF_snow_end        = d_SRF_end['snow_beg']        ; SRF_snow_end        = SRF_snow_end       .where (SRF_snow_end        < 1E15, 0.)
SRF_lakeres_end     = d_SRF_end['lakeres']         ; SRF_lakeres_end     = SRF_lakeres_end    .where (SRF_lakeres_end     < 1E15, 0.)

if LMDZ :
    SRF_tot_watveg_beg  = lmdz.geo2point (SRF_tot_watveg_beg , dim1D='cell')
    SRF_tot_watsoil_beg = lmdz.geo2point (SRF_tot_watsoil_beg, dim1D='cell')
    SRF_snow_beg        = lmdz.geo2point (SRF_snow_beg       , dim1D='cell')
    SRF_lakeres_beg     = lmdz.geo2point (SRF_lakeres_beg    , dim1D='cell')
    SRF_tot_watveg_end  = lmdz.geo2point (SRF_tot_watveg_end , dim1D='cell')
    SRF_tot_watsoil_end = lmdz.geo2point (SRF_tot_watsoil_end, dim1D='cell')
    SRF_snow_end        = lmdz.geo2point (SRF_snow_end       , dim1D='cell')
    SRF_lakeres_end     = lmdz.geo2point (SRF_lakeres_end    , dim1D='cell')
  
if ICO :
    SRF_tot_watveg_beg  = SRF_tot_watveg_beg  [SRF_beg_keysort]
    SRF_tot_watsoil_beg = SRF_tot_watsoil_beg [SRF_beg_keysort]
    SRF_snow_beg        = SRF_snow_beg        [SRF_beg_keysort]
    SRF_lakeres_beg     = SRF_lakeres_beg     [SRF_beg_keysort]
    SRF_tot_watveg_end  = SRF_tot_watveg_end  [SRF_end_keysort]
    SRF_tot_watsoil_end = SRF_tot_watsoil_end [SRF_end_keysort]
    SRF_snow_end        = SRF_snow_end        [SRF_end_keysort]
    SRF_lakeres_end     = SRF_lakeres_end     [SRF_end_keysort]

# Stock dSoilHum dInterce dSWE dStream dFastR dSlowR dLake dPond dFlood

SRF_wat_beg = SRF_tot_watveg_beg + SRF_tot_watsoil_beg + SRF_snow_beg
SRF_wat_end = SRF_tot_watveg_end + SRF_tot_watsoil_end + SRF_snow_end

echo ( '\n====================================================================================' )
print ('Computing integrals')

print ( ' 1/8', end='' ) ; sys.stdout.flush ()
SRF_mas_watveg_beg   = SRF_stock_int ( SRF_tot_watveg_beg  )
print ( ' 2/8', end='' ) ; sys.stdout.flush ()
SRF_mas_watsoil_beg  = SRF_stock_int ( SRF_tot_watsoil_beg )
print ( ' 3/8', end='' ) ; sys.stdout.flush ()
SRF_mas_snow_beg     = SRF_stock_int ( SRF_snow_beg        )
print ( ' 4/8', end='' ) ; sys.stdout.flush ()
SRF_mas_lake_beg     = ONE_stock_int ( SRF_lakeres_beg     )
print ( ' 5/8', end='' ) ; sys.stdout.flush ()

SRF_mas_watveg_end   = SRF_stock_int ( SRF_tot_watveg_end  )
print ( ' 6/8', end='' ) ; sys.stdout.flush ()
SRF_mas_watsoil_end  = SRF_stock_int ( SRF_tot_watsoil_end )
print ( ' 7/8', end='' ) ; sys.stdout.flush ()
SRF_mas_snow_end     = SRF_stock_int ( SRF_snow_end        )
print ( ' 8/8', end='' ) ; sys.stdout.flush ()
SRF_mas_lake_end     = ONE_stock_int ( SRF_lakeres_end     )

print (' -- ') ; sys.stdout.flush ()

dSRF_mas_watveg   = Sprec ( [SRF_mas_watveg_end , -SRF_mas_watveg_beg] )
dSRF_mas_watsoil  = Sprec ( [SRF_mas_watsoil_end, -SRF_mas_watsoil_beg] )
dSRF_mas_snow     = Sprec ( [SRF_mas_snow_end   , -SRF_mas_snow_beg] )
dSRF_mas_lake     = Sprec ( [SRF_mas_lake_end   , -SRF_mas_lake_beg] )

echo ( '------------------------------------------------------------------------------------' )
echo ( f'\nSurface reservoirs  -- {Title} ')
echo ( f'SRF_mas_watveg_beg   = {SRF_mas_watveg_beg :12.6e} kg | SRF_mas_watveg_end   = {SRF_mas_watveg_end :12.6e} kg ' )
echo ( f'SRF_mas_watsoil_beg  = {SRF_mas_watsoil_beg:12.6e} kg | SRF_mas_watsoil_end  = {SRF_mas_watsoil_end:12.6e} kg ' )
echo ( f'SRF_mas_snow_beg     = {SRF_mas_snow_beg   :12.6e} kg | SRF_mas_snow_end     = {SRF_mas_snow_end   :12.6e} kg ' )
echo ( f'SRF_mas_lake_beg     = {SRF_mas_lake_beg   :12.6e} kg | SRF_mas_lake_end     = {SRF_mas_lake_end   :12.6e} kg ' )

prtFlux ( 'dMass (watveg) ', dSRF_mas_watveg , 'e' , True )
prtFlux ( 'dMass (watsoil)', dSRF_mas_watsoil, 'e' , True )
prtFlux ( 'dMass (snow)   ', dSRF_mas_snow   , 'e' , True )
prtFlux ( 'dMass (lake)   ', dSRF_mas_lake   , 'e' , True )

SRF_mas_wat_beg = Sprec ( [SRF_mas_watveg_beg , SRF_mas_watsoil_beg, SRF_mas_snow_beg] )
SRF_mas_wat_end = Sprec ( [SRF_mas_watveg_end , SRF_mas_watsoil_end, SRF_mas_snow_end] )

dSRF_mas_wat    = Sprec ( [+SRF_mas_watveg_end , +SRF_mas_watsoil_end, +SRF_mas_snow_end, 
                           -SRF_mas_watveg_beg , -SRF_mas_watsoil_beg, -SRF_mas_snow_beg]  )

echo ( '------------------------------------------------------------------------------------' )
echo ( f'Water content in surface  -- {Title} ' )
echo ( f'SRF_mas_wat_beg   = {SRF_mas_wat_beg:12.6e} kg | SRF_mas_wat_end  = {SRF_mas_wat_end:12.6e} kg ')
prtFlux ( 'dMass (water srf)', dSRF_mas_wat, 'e', True )

echo ( '------------------------------------------------------------------------------------' )
echo ( 'Water content in  ATM + SRF + RUN + LAKE' )
echo ( 'mas_wat_beg = {:12.6e} kg | mas_wat_end = {:12.6e} kg '.
           format (DYN_mas_wat_beg + ATM_mas_sno_beg + RUN_mas_wat_beg + SRF_mas_wat_beg + SRF_mas_lake_beg ,
                   DYN_mas_wat_end + ATM_mas_sno_end + RUN_mas_wat_end + SRF_mas_wat_end + SRF_mas_lake_end ) )
prtFlux ( 'dMass (water atm+srf+run+lake)', dDYN_mas_wat + dATM_mas_sno + dRUN_mas_wat + dSRF_mas_wat + dSRF_mas_lake, 'e', True)

echo ( '\n====================================================================================' )
echo ( f'-- ATM Fluxes  -- {Title} ' )

if ATM_HIS == 'latlon' :
    echo ( ' latlon case' )
    ATM_wbilo_oce   = lmdz.geo2point ( rprec (d_ATM_his ['wbilo_oce']), dim1D='cell' )
    ATM_wbilo_sic   = lmdz.geo2point ( rprec (d_ATM_his ['wbilo_sic']), dim1D='cell' )
    ATM_wbilo_ter   = lmdz.geo2point ( rprec (d_ATM_his ['wbilo_ter']), dim1D='cell' )
    ATM_wbilo_lic   = lmdz.geo2point ( rprec (d_ATM_his ['wbilo_lic']), dim1D='cell' )
    ATM_runofflic   = lmdz.geo2point ( rprec (d_ATM_his ['runofflic']), dim1D='cell' )
    ATM_fqcalving   = lmdz.geo2point ( rprec (d_ATM_his ['fqcalving']), dim1D='cell' )
    ATM_fqfonte     = lmdz.geo2point ( rprec (d_ATM_his ['fqfonte']  ), dim1D='cell' )
    ATM_precip      = lmdz.geo2point ( rprec (d_ATM_his ['precip']   ), dim1D='cell' )
    ATM_snowf       = lmdz.geo2point ( rprec (d_ATM_his ['snow']     ), dim1D='cell' )
    ATM_evap        = lmdz.geo2point ( rprec (d_ATM_his ['evap']     ), dim1D='cell' )
    ATM_wevap_ter   = lmdz.geo2point ( rprec (d_ATM_his ['wevap_ter']), dim1D='cell' )
    ATM_wevap_oce   = lmdz.geo2point ( rprec (d_ATM_his ['wevap_oce']), dim1D='cell' )
    ATM_wevap_lic   = lmdz.geo2point ( rprec (d_ATM_his ['wevap_lic']), dim1D='cell' )
    ATM_wevap_sic   = lmdz.geo2point ( rprec (d_ATM_his ['wevap_sic']), dim1D='cell' )
    ATM_wrain_ter   = lmdz.geo2point ( rprec (d_ATM_his ['wrain_ter']), dim1D='cell' )
    ATM_wrain_oce   = lmdz.geo2point ( rprec (d_ATM_his ['wrain_oce']), dim1D='cell' )
    ATM_wrain_lic   = lmdz.geo2point ( rprec (d_ATM_his ['wrain_lic']), dim1D='cell' )
    ATM_wrain_sic   = lmdz.geo2point ( rprec (d_ATM_his ['wrain_sic']), dim1D='cell' )
    ATM_wsnow_ter   = lmdz.geo2point ( rprec (d_ATM_his ['wsnow_ter']), dim1D='cell' )
    ATM_wsnow_oce   = lmdz.geo2point ( rprec (d_ATM_his ['wsnow_oce']), dim1D='cell' )
    ATM_wsnow_lic   = lmdz.geo2point ( rprec (d_ATM_his ['wsnow_lic']), dim1D='cell' )
    ATM_wsnow_sic   = lmdz.geo2point ( rprec (d_ATM_his ['wsnow_sic']), dim1D='cell' )
    ATM_runofflic   = lmdz.geo2point ( rprec (d_ATM_his ['runofflic']), dim1D='cell' )
    echo ( f'End of LATLON case')
    
if ATM_HIS == 'ico' :
    echo (' ico case')
    ATM_wbilo_oce   = rprec (d_ATM_his ['wbilo_oce'])[..., ATM_his_keysort]
    ATM_wbilo_sic   = rprec (d_ATM_his ['wbilo_sic'])[..., ATM_his_keysort]
    ATM_wbilo_ter   = rprec (d_ATM_his ['wbilo_ter'])[..., ATM_his_keysort]
    ATM_wbilo_lic   = rprec (d_ATM_his ['wbilo_lic'])[..., ATM_his_keysort]
    ATM_runofflic   = rprec (d_ATM_his ['runofflic'])[..., ATM_his_keysort]
    ATM_fqcalving   = rprec (d_ATM_his ['fqcalving'])[..., ATM_his_keysort]
    ATM_fqfonte     = rprec (d_ATM_his ['fqfonte']  )[..., ATM_his_keysort]
    ATM_precip      = rprec (d_ATM_his ['precip']   )[..., ATM_his_keysort]
    ATM_snowf       = rprec (d_ATM_his ['snow']     )[..., ATM_his_keysort]
    ATM_evap        = rprec (d_ATM_his ['evap']     )[..., ATM_his_keysort]
    ATM_wevap_ter   = rprec (d_ATM_his ['wevap_ter'])[..., ATM_his_keysort]
    ATM_wevap_oce   = rprec (d_ATM_his ['wevap_oce'])[..., ATM_his_keysort]
    ATM_wevap_lic   = rprec (d_ATM_his ['wevap_lic'])[..., ATM_his_keysort]
    ATM_wevap_sic   = rprec (d_ATM_his ['wevap_sic'])[..., ATM_his_keysort]
    ATM_runofflic   = rprec (d_ATM_his ['runofflic'])[..., ATM_his_keysort]
    ATM_wevap_ter   = rprec (d_ATM_his ['wevap_ter'])[..., ATM_his_keysort]
    ATM_wevap_oce   = rprec (d_ATM_his ['wevap_oce'])[..., ATM_his_keysort]
    ATM_wevap_lic   = rprec (d_ATM_his ['wevap_lic'])[..., ATM_his_keysort]
    ATM_wevap_sic   = rprec (d_ATM_his ['wevap_sic'])[..., ATM_his_keysort]
    ATM_wrain_ter   = rprec (d_ATM_his ['wrain_ter'])[..., ATM_his_keysort]
    ATM_wrain_oce   = rprec (d_ATM_his ['wrain_oce'])[..., ATM_his_keysort]
    ATM_wrain_lic   = rprec (d_ATM_his ['wrain_lic'])[..., ATM_his_keysort]
    ATM_wrain_sic   = rprec (d_ATM_his ['wrain_sic'])[..., ATM_his_keysort]
    ATM_wsnow_ter   = rprec (d_ATM_his ['wsnow_ter'])[..., ATM_his_keysort]
    ATM_wsnow_oce   = rprec (d_ATM_his ['wsnow_oce'])[..., ATM_his_keysort]
    ATM_wsnow_lic   = rprec (d_ATM_his ['wsnow_lic'])[..., ATM_his_keysort]
    ATM_wsnow_sic   = rprec (d_ATM_his ['wsnow_sic'])[..., ATM_his_keysort]
    echo ( f'End of ico case ')

echo ( 'ATM wprecip_oce' )
ATM_wprecip_oce = ATM_wrain_oce + ATM_wsnow_oce
ATM_wprecip_ter = ATM_wrain_ter + ATM_wsnow_ter
ATM_wprecip_sic = ATM_wrain_sic + ATM_wsnow_sic
ATM_wprecip_lic = ATM_wrain_lic + ATM_wsnow_lic

ATM_wbilo       = ATM_wbilo_oce   + ATM_wbilo_sic   + ATM_wbilo_ter   + ATM_wbilo_lic
ATM_wevap       = ATM_wevap_oce   + ATM_wevap_sic   + ATM_wevap_ter   + ATM_wevap_lic
ATM_wprecip     = ATM_wprecip_oce + ATM_wprecip_sic + ATM_wprecip_ter + ATM_wprecip_lic
ATM_wsnow       = ATM_wsnow_oce   + ATM_wsnow_sic   + ATM_wsnow_ter   + ATM_wsnow_lic
ATM_wrain       = ATM_wrain_oce   + ATM_wrain_sic   + ATM_wrain_ter   + ATM_wrain_lic
ATM_wemp        = ATM_wevap - ATM_wprecip
ATM_emp         = ATM_evap  - ATM_precip

ATM_wprecip_sea = ATM_wprecip_oce + ATM_wprecip_sic
ATM_wsnow_sea   = ATM_wsnow_oce   + ATM_wsnow_sic
ATM_wrain_sea   = ATM_wrain_oce   + ATM_wrain_sic
ATM_wbilo_sea   = ATM_wbilo_oce   + ATM_wbilo_sic
ATM_wevap_sea   = ATM_wevap_sic   + ATM_wevap_oce

ATM_wemp_ter    = ATM_wevap_ter - ATM_wprecip_ter
ATM_wemp_oce    = ATM_wevap_oce - ATM_wprecip_oce
ATM_wemp_sic    = ATM_wevap_sic - ATM_wprecip_sic
ATM_wemp_lic    = ATM_wevap_lic - ATM_wprecip_lic
ATM_wemp_sea    = ATM_wevap_sic - ATM_wprecip_oce

if RUN_HIS == 'latlon' :
    echo ( f'RUN costalflow Grille LATLON' )
    if TestInterp :
         echo ( f'RUN runoff TestInterp' )
         RUN_runoff      = lmdz.geo2point ( rprec (d_RUN_his ['runoff_contfrac_interp']  )   , dim1D='cell' )
         RUN_drainage    = lmdz.geo2point ( rprec (d_RUN_his ['drainage_contfrac_interp'])   , dim1D='cell' )
    else : 
        echo ( f'RUN runoff' )
        RUN_runoff      = lmdz.geo2point ( rprec (d_RUN_his ['runoff']         ), dim1D='cell' )
        RUN_drainage    = lmdz.geo2point ( rprec (d_RUN_his ['drainage']       ), dim1D='cell' )
        
    RUN_coastalflow     = lmdz.geo2point ( rprec (d_RUN_his ['coastalflow']    ), dim1D='cell' ) 
    RUN_riverflow       = lmdz.geo2point ( rprec (d_RUN_his ['riverflow']      ), dim1D='cell' )
    RUN_riversret       = lmdz.geo2point ( rprec (d_RUN_his ['riversret']      ), dim1D='cell' )
    RUN_coastalflow_cpl = lmdz.geo2point ( rprec (d_RUN_his ['coastalflow_cpl']), dim1D='cell' ) 
    RUN_riverflow_cpl   = lmdz.geo2point ( rprec (d_RUN_his ['riverflow_cpl']  ), dim1D='cell' )

if RUN_HIS == 'ico' :
    echo ( f'RUN costalflow Grille ICO' )
    RUN_coastalflow =  rprec (d_RUN_his ['coastalflow'])[..., RUN_his_keysort]
    RUN_riverflow   =  rprec (d_RUN_his ['riverflow']  )[..., RUN_his_keysort]
    RUN_runoff      =  rprec (d_RUN_his ['runoff']     )[..., RUN_his_keysort]
    RUN_drainage    =  rprec (d_RUN_his ['drainage']   )[..., RUN_his_keysort]
    RUN_riversret   =  rprec (d_RUN_his ['riversret']  )[..., RUN_his_keysort]
    
    RUN_coastalflow_cpl = rprec (d_RUN_his ['coastalflow_cpl'])[..., RUN_his_keysort]
    RUN_riverflow_cpl   = rprec (d_RUN_his ['riverflow_cpl']  )[..., RUN_his_keysort]

Step = 0

if SRF_HIS == 'latlon' :
    if TestInterp :
         echo ( f'SRF rain TestInterp' )
         SRF_rain     = lmdz.geo2point ( rprec (d_SRF_his ['rain_contfrac_interp'] ), dim1D='cell')
         SRF_evap     = lmdz.geo2point ( rprec (d_SRF_his ['evap_contfrac_interp'] ), dim1D='cell')
         SRF_snowf    = lmdz.geo2point ( rprec (d_SRF_his ['snow_contfrac_interp'] ), dim1D='cell')
         SRF_subli    = lmdz.geo2point ( rprec (d_SRF_his ['subli_contfrac_interp']), dim1D='cell')
         SRF_transpir = lmdz.geo2point ( rprec (d_SRF_his ['transpir_contfrac_interp']).sum(dim='veget'), dim1D='cell' )
         SRF_rain.attrs     = d_SRF_his ['rain_contfrac_interp'].attrs
         SRF_evap.attrs     = d_SRF_his ['evap_contfrac_interp'].attrs
         SRF_snowf.attrs    = d_SRF_his ['snow_contfrac_interp'].attrs
         SRF_subli.attrs    = d_SRF_his ['subli_contfrac_interp'].attrs
         SRF_transpir.attrs = d_SRF_his ['transpir_contfrac_interp'].attrs
    else : 
        echo ( f'SRF rain' )
        SRF_rain     = lmdz.geo2point ( rprec (d_SRF_his ['rain'] ) , dim1D='cell')
        SRF_evap     = lmdz.geo2point ( rprec (d_SRF_his ['evap'] ) , dim1D='cell')
        SRF_snowf    = lmdz.geo2point ( rprec (d_SRF_his ['snowf']) , dim1D='cell')
        SRF_subli    = lmdz.geo2point ( rprec (d_SRF_his ['subli']) , dim1D='cell')
        SRF_transpir = lmdz.geo2point ( rprec (d_SRF_his ['transpir']).sum(dim='veget'), dim1D='cell' )

if SRF_HIS == 'ico' :
    echo ( f'SRF rain')
    SRF_rain     = rprec (d_SRF_his ['rain'] )[..., SRF_his_keysort]
    SRF_evap     = rprec (d_SRF_his ['evap'] )[..., SRF_his_keysort]
    SRF_snowf    = rprec (d_SRF_his ['snowf'])[..., SRF_his_keysort]
    SRF_subli    = rprec (d_SRF_his ['subli'])[..., SRF_his_keysort]
    SRF_transpir = rprec (d_SRF_his ['transpir']).sum(dim='veget')[..., SRF_his_keysort]

echo ( f'SRF emp' )
SRF_transpir.attrs['units'] = d_SRF_his ['transpir'].attrs['units']
SRF_emp      = SRF_evap - SRF_rain - SRF_snowf ; SRF_emp.attrs['units'] = SRF_rain.attrs['units']
    
## Correcting units of SECHIBA variables
def mmd2SI ( Var ) :
    '''Change unit from mm/d or m^3/s to kg/s if needed'''
    if 'units' in VarT.attrs : 
        if VarT.attrs['units'] in ['m^3/s', 'm3/s', 'm3.s-1',] :
            VarT.values = VarT.values  * ATM_rho                 ;  VarT.attrs['units'] = 'kg/s'
        if VarT.attrs['units'] == 'mm/d' :
            VarT.values = VarT.values  * ATM_rho * (1e-3/86400.) ;  VarT.attrs['units'] = 'kg/s'
        if VarT.attrs['units'] in ['m^3', 'm3', ] :
            VarT.values = VarT.values  * ATM_rho                 ;  VarT.attrs['units'] = 'kg'
            
for var in [ 'runoff', 'drainage', 'riversret', 'coastalflow', 'riverflow', 'coastalflow_cpl', 'riverflow_cpl' ] :
    VarT = locals()['RUN_' + var]
    mmd2SI (VarT)

for var in ['evap', 'snowf', 'subli', 'transpir', 'rain', 'emp' ] :
    VarT = locals()['SRF_' + var]
    mmd2SI (VarT)

echo ( f'RUN input' )
RUN_input  = RUN_runoff      + RUN_drainage
RUN_output = RUN_coastalflow + RUN_riverflow

echo ( f'ATM flw_wbilo' )
ATM_flx_wbilo       = ATM_flux_int ( ATM_wbilo      )
ATM_flx_wevap       = ATM_flux_int ( ATM_wevap      )
ATM_flx_wprecip     = ATM_flux_int ( ATM_wprecip    )
ATM_flx_wsnow       = ATM_flux_int ( ATM_wsnow      )
ATM_flx_wrain       = ATM_flux_int ( ATM_wrain      )
ATM_flx_wemp        = ATM_flux_int ( ATM_wemp       )

ATM_flx_wbilo_lic   = ATM_flux_int ( ATM_wbilo_lic  )
ATM_flx_wbilo_oce   = ATM_flux_int ( ATM_wbilo_oce  )
ATM_flx_wbilo_sea   = ATM_flux_int ( ATM_wbilo_sea  )
ATM_flx_wbilo_sic   = ATM_flux_int ( ATM_wbilo_sic  )
ATM_flx_wbilo_ter   = ATM_flux_int ( ATM_wbilo_ter  )
ATM_flx_calving     = ATM_flux_int ( ATM_fqcalving  )
ATM_flx_fqfonte     = ATM_flux_int ( ATM_fqfonte    )

LIC_flx_calving     = LIC_flux_int ( ATM_fqcalving  )
LIC_flx_fqfonte     = LIC_flux_int ( ATM_fqfonte    )

echo ( f'ATM flx precip' )
ATM_flx_precip      = ATM_flux_int ( ATM_precip     )
ATM_flx_snowf       = ATM_flux_int ( ATM_snowf      )
ATM_flx_evap        = ATM_flux_int ( ATM_evap       )
ATM_flx_runlic      = ATM_flux_int ( ATM_runofflic  )

LIC_flx_precip      = LIC_flux_int ( ATM_precip     )
LIC_flx_snowf       = LIC_flux_int ( ATM_snowf      )
LIC_flx_evap        = LIC_flux_int ( ATM_evap       )
LIC_flx_runlic      = LIC_flux_int ( ATM_runofflic  )

echo ( f'ATM flx_wrain_ter' )
ATM_flx_wrain_ter    = ATM_flux_int ( ATM_wrain_ter )
ATM_flx_wrain_oce    = ATM_flux_int ( ATM_wrain_oce )
ATM_flx_wrain_lic    = ATM_flux_int ( ATM_wrain_lic )
ATM_flx_wrain_sic    = ATM_flux_int ( ATM_wrain_sic )
ATM_flx_wrain_sea    = ATM_flux_int ( ATM_wrain_sea )

ATM_flx_wsnow_ter    = ATM_flux_int ( ATM_wsnow_ter )
ATM_flx_wsnow_oce    = ATM_flux_int ( ATM_wsnow_oce )
ATM_flx_wsnow_lic    = ATM_flux_int ( ATM_wsnow_lic )
ATM_flx_wsnow_sic    = ATM_flux_int ( ATM_wsnow_sic )
ATM_flx_wsnow_sea    = ATM_flux_int ( ATM_wsnow_sea )

echo ( f'ATM flx_evap_ter' ) 
ATM_flx_wevap_ter    = ATM_flux_int ( ATM_wevap_ter )
ATM_flx_wevap_oce    = ATM_flux_int ( ATM_wevap_oce )
ATM_flx_wevap_lic    = ATM_flux_int ( ATM_wevap_lic )
ATM_flx_wevap_sic    = ATM_flux_int ( ATM_wevap_sic )
ATM_flx_wevap_sea    = ATM_flux_int ( ATM_wevap_sea )
ATM_flx_wprecip_lic  = ATM_flux_int ( ATM_wprecip_lic )
ATM_flx_wprecip_oce  = ATM_flux_int ( ATM_wprecip_oce )
ATM_flx_wprecip_sic  = ATM_flux_int ( ATM_wprecip_sic )
ATM_flx_wprecip_ter  = ATM_flux_int ( ATM_wprecip_ter )
ATM_flx_wprecip_sea  = ATM_flux_int ( ATM_wprecip_sea )
ATM_flx_wemp_lic     = ATM_flux_int ( ATM_wemp_lic )
ATM_flx_wemp_oce     = ATM_flux_int ( ATM_wemp_oce )
ATM_flx_wemp_sic     = ATM_flux_int ( ATM_wemp_sic )
ATM_flx_wemp_ter     = ATM_flux_int ( ATM_wemp_ter )
ATM_flx_wemp_sea     = ATM_flux_int ( ATM_wemp_sea )

ATM_flx_emp          = ATM_flux_int ( ATM_emp )

echo ( f'RUN flx_coastal' )
RUN_flx_coastal     = ONE_flux_int ( RUN_coastalflow)
echo ( f'RUN flx_river' )
RUN_flx_river       = ONE_flux_int ( RUN_riverflow  )
echo ( f'RUN flx_coastal_cpl' )
RUN_flx_coastal_cpl = ONE_flux_int ( RUN_coastalflow_cpl)
echo ( f'RUN flx_river_cpl' )
RUN_flx_river_cpl   = ONE_flux_int ( RUN_riverflow_cpl  )
echo ( f'RUN flx_drainage' )
RUN_flx_drainage    = SRF_flux_int ( RUN_drainage   )
echo ( f'RUN flx_riversset' )
RUN_flx_riversret   = SRF_flux_int ( RUN_riversret  )
echo ( f'RUN flx_runoff' )
RUN_flx_runoff      = SRF_flux_int ( RUN_runoff     )
echo ( f'RUN flx_input' )
RUN_flx_input       = SRF_flux_int ( RUN_input      )
echo ( f'RUN flx_output' )
RUN_flx_output      = ONE_flux_int ( RUN_output     )

echo ( f'RUN flx_bil' ) ; Step += 1
#RUN_flx_bil    = RUN_flx_input   - RUN_flx_output
#RUN_flx_rivcoa = RUN_flx_coastal + RUN_flx_river

RUN_flx_bil    = ONE_flux_int ( RUN_input       - RUN_output)
RUN_flx_rivcoa = ONE_flux_int ( RUN_coastalflow + RUN_riverflow)

prtFlux ('wbilo_oce            ', ATM_flx_wbilo_oce     , 'f' )          
prtFlux ('wbilo_sic            ', ATM_flx_wbilo_sic     , 'f' )          
prtFlux ('wbilo_sic+oce        ', ATM_flx_wbilo_sea     , 'f' )          
prtFlux ('wbilo_ter            ', ATM_flx_wbilo_ter     , 'f' )          
prtFlux ('wbilo_lic            ', ATM_flx_wbilo_lic     , 'f' )          
prtFlux ('Sum wbilo_*          ', ATM_flx_wbilo         , 'f', True)  
prtFlux ('E-P                  ', ATM_flx_emp           , 'f', True)  
prtFlux ('calving              ', ATM_flx_calving       , 'f' ) 
prtFlux ('fqfonte              ', ATM_flx_fqfonte       , 'f' )       
prtFlux ('precip               ', ATM_flx_precip        , 'f' )       
prtFlux ('snowf                ', ATM_flx_snowf         , 'f' )        
prtFlux ('evap                 ', ATM_flx_evap          , 'f' )
prtFlux ('runoff lic           ', ATM_flx_runlic        , 'f' )

prtFlux ('ATM_flx_wevap*       ', ATM_flx_wevap         , 'f' )
prtFlux ('ATM_flx_wrain*       ', ATM_flx_wrain         , 'f' )
prtFlux ('ATM_flx_wsnow*       ', ATM_flx_wsnow         , 'f' )
prtFlux ('ATM_flx_wprecip*     ', ATM_flx_wprecip       , 'f' )
prtFlux ('ATM_flx_wemp*        ', ATM_flx_wemp          , 'f', True )

prtFlux ('ERROR evap           ', ATM_flx_wevap   - ATM_flx_evap  , 'e', True )
prtFlux ('ERROR precip         ', ATM_flx_wprecip - ATM_flx_precip, 'e', True )
prtFlux ('ERROR snow           ', ATM_flx_wsnow   - ATM_flx_snowf , 'e', True )
prtFlux ('ERROR emp            ', ATM_flx_wemp    - ATM_flx_emp   , 'e', True )


echo ( '\n====================================================================================' )
echo ( f'-- RUNOFF Fluxes  -- {Title} ' )
prtFlux ('coastalflow   ', RUN_flx_coastal    , 'f' ) 
prtFlux ('riverflow     ', RUN_flx_river      , 'f' )        
prtFlux ('coastal_cpl   ', RUN_flx_coastal_cpl, 'f' )  
prtFlux ('riverf_cpl    ', RUN_flx_river_cpl  , 'f' )    
prtFlux ('river+coastal ', RUN_flx_rivcoa     , 'f' )   
prtFlux ('drainage      ', RUN_flx_drainage   , 'f' )   
prtFlux ('riversret     ', RUN_flx_riversret  , 'f' )   
prtFlux ('runoff        ', RUN_flx_runoff     , 'f' )   
prtFlux ('river in      ', RUN_flx_input      , 'f' )   
prtFlux ('river out     ', RUN_flx_output     , 'f' )   
prtFlux ('river bil     ', RUN_flx_bil        , 'f' )   

ATM_flx_budget = -ATM_flx_wbilo + ATM_flx_calving + ATM_flx_runlic #+ ATM_flx_fqfonte + RUN_flx_river


echo ('')
#echo ('  Global        {:12.3e} kg | {:12.4f} Sv | {:12.4f} m '.format ( ATM_flx_budget , ATM_flx_budget / dtime_sec*1E-9, ATM_flx_budget /ATM_aire_sea_tot/ATM_rho ))

#echo ('  E-P-R         {:12.3e} kg | {:12.4e} Sv | {:12.4f} m '.format ( ATM_flx_emp      , ATM_flx_emp      / dtime_sec*1E-6/ATM_rho, ATM_flx_emp      /ATM_aire_sea_tot/ATM_rho ))

ATM_flx_toSRF = -ATM_flx_wbilo_ter

echo (' ')
echo ( '\n====================================================================================' )
echo ( f'--  Atmosphere  -- {Title} ' )
echo ( f'Mass begin = {DYN_mas_wat_beg:12.6e} kg | Mass end = {DYN_mas_wat_end:12.6e} kg' )
prtFlux ( 'dmass (atm)  = ', dDYN_mas_wat , 'e', True )
prtFlux ( 'Sum wbilo_*  = ', ATM_flx_wbilo, 'e', True )
prtFlux ( 'E-P          = ', ATM_flx_emp  , 'e', True )
echo ( ' ' )
prtFlux ( 'Water loss atm', ATM_flx_wbilo - dDYN_mas_wat, 'f', True )
echo ( 'Water loss atm = {:12.3e} (rel)  '.format ( (ATM_flx_wbilo - dDYN_mas_wat)/dDYN_mas_wat  ) )

echo (' ')
prtFlux ( 'Water loss atm', ATM_flx_emp  - dDYN_mas_wat , 'f', True )
echo ( 'Water loss atm = {:12.3e} (rel)  '.format ( (ATM_flx_emp-dDYN_mas_wat)/dDYN_mas_wat  ) )
echo (' ')

echo (' ')
echo ( '\n====================================================================================' )

LIC_flx_budget1 = Sprec ( [-ATM_flx_wemp_lic  , -LIC_flx_calving , -LIC_flx_fqfonte] )
LIC_flx_budget2 = Sprec ( [-ATM_flx_wbilo_lic , -LIC_flx_calving , -LIC_flx_fqfonte] )
LIC_flx_budget3 = Sprec ( [-ATM_flx_wbilo_lic , -LIC_flx_runlic] )
LIC_flx_budget4 = Sprec ( [-ATM_flx_wemp_lic  , -LIC_flx_runlic] )

echo ( f'--  LIC  -- {Title} ' )
echo ( f'Mass total   begin = {LIC_mas_wat_beg    :12.6e} kg | Mass end = {LIC_mas_wat_end    :12.6e} kg' )
echo ( f'Mass snow    begin = {LIC_mas_sno_beg    :12.6e} kg | Mass end = {LIC_mas_sno_end    :12.6e} kg' )
echo ( f'Mass qs      begin = {LIC_mas_qs_beg     :12.6e} kg | Mass end = {LIC_mas_qs_end     :12.6e} kg' )
echo ( f'Mass runlic0 begin = {LIC_mas_runlic0_beg:12.6e} kg | Mass end = {LIC_mas_runlic0_end:12.6e} kg' )
prtFlux ( 'dmass (LIC sno)       ', dLIC_mas_sno          , 'f', True, width=45 )
prtFlux ( 'dmass (LIC qs)        ', dLIC_mas_qs           , 'e', True, width=45 )
prtFlux ( 'dmass (LIC wat)       ', dLIC_mas_wat          , 'f', True, width=45 )
prtFlux ( 'dmass (LIC runlic0)   ', dLIC_mas_runlic0      , 'e', True, width=45 )
prtFlux ( 'dmass (LIC total)     ', dLIC_mas_wat          , 'e', True, width=45 )
prtFlux ( 'LIC ATM_flx_wemp_lic  ', ATM_flx_wemp_lic      , 'f', True, width=45 )
prtFlux ( 'LIC LIC_flx_fqfonte   ', LIC_flx_fqfonte       , 'f', True, width=45 )
prtFlux ( 'LIC LIC_flx_calving   ', LIC_flx_calving       , 'f', True, width=45 )
prtFlux ( 'LIC LIC_flx_runofflic ', LIC_flx_runlic        , 'f', True, width=45 )
prtFlux ( 'LIC fqfonte + calving ', LIC_flx_calving+LIC_flx_fqfonte , 'f', True, width=45 )
prtFlux ( 'LIC fluxes 1 ( wemp_lic   - fqcalving - fqfonte)) ', LIC_flx_budget1              , 'f', True, width=45 )
prtFlux ( 'LIC fluxes 2 (-wbilo_lic - fqcalving - fqfonte)   ', LIC_flx_budget2              , 'f', True, width=45 )
prtFlux ( 'LIC fluxes 3 (-wbilo_lic - runofflic*frac_lic)    ', LIC_flx_budget3              , 'f', True, width=45 )
prtFlux ( 'LIC fluxes 3 ( wemp_lic  - runofflic*frac_lic)    ', LIC_flx_budget4              , 'f', True, width=45 )
prtFlux ( 'LIC error 1                                       ', LIC_flx_budget1-dLIC_mas_wat , 'e', True, width=45 )
prtFlux ( 'LIC error 2                                       ', LIC_flx_budget2-dLIC_mas_wat , 'e', True, width=45 )
prtFlux ( 'LIC error 3                                       ', LIC_flx_budget3-dLIC_mas_wat , 'e', True, width=45 )
echo ( 'LIC error (wevap - precip*frac_lic  - fqcalving - fqfonte)    = {:12.4e} (rel) '.format ( (LIC_flx_budget1-dLIC_mas_wat)/dLIC_mas_wat) )
echo ( 'LIC error (-wbilo_lic - fqcalving - fqfonte)                  = {:12.4e} (rel) '.format ( (LIC_flx_budget2-dLIC_mas_wat)/dLIC_mas_wat) )
echo ( 'LIC error (-wbilo_lic - runofflic*frac_lic)                   = {:12.4e} (rel) '.format ( (LIC_flx_budget3-dLIC_mas_wat)/dLIC_mas_wat) )

echo ( '\n====================================================================================' )
echo ( f'-- SECHIBA fluxes  -- {Title} ' )

SRF_flx_rain     = SRF_flux_int ( SRF_rain     )
SRF_flx_evap     = SRF_flux_int ( SRF_evap     )
SRF_flx_snowf    = SRF_flux_int ( SRF_snowf    )
SRF_flx_subli    = SRF_flux_int ( SRF_subli    )
SRF_flx_transpir = SRF_flux_int ( SRF_transpir )
SRF_flx_emp      = SRF_flux_int ( SRF_emp      )

RUN_flx_torouting   = SRF_flux_int  ( RUN_runoff       + RUN_drainage)
RUN_flx_fromrouting = ONE_flux_int  ( RUN_coastalflow + RUN_riverflow )

SRF_flx_all =  SRF_flux_int  ( SRF_rain + SRF_snowf - SRF_evap - RUN_runoff - RUN_drainage )

prtFlux ('rain         ', SRF_flx_rain       , 'f' )
prtFlux ('evap         ', SRF_flx_evap       , 'f' )
prtFlux ('snowf        ', SRF_flx_snowf      , 'f' )
prtFlux ('E-P          ', SRF_flx_emp        , 'f' )
prtFlux ('subli        ', SRF_flx_subli      , 'f' )
prtFlux ('transpir     ', SRF_flx_transpir   , 'f' )
prtFlux ('to routing   ', RUN_flx_torouting  , 'f' )
prtFlux ('budget       ', SRF_flx_all        , 'f', small=True )

echo ( '\n------------------------------------------------------------------------------------' )
echo ( 'Water content in surface ' )
echo ( f'SRF_mas_wat_beg  = {SRF_mas_wat_beg:12.6e} kg | SRF_mas_wat_end  = {SRF_mas_wat_end:12.6e} kg ' )
prtFlux ( 'dMass (water srf)', dSRF_mas_wat, 'e', small=True)
prtFlux ( 'Error            ',  SRF_flx_all-dSRF_mas_wat, 'e', small=True )
echo ( 'dMass (water srf) = {:12.4e} (rel)   '.format ( (SRF_flx_all-dSRF_mas_wat)/dSRF_mas_wat) )

echo ( '\n====================================================================================' )
echo ( f'-- Check ATM vs. SRF -- {Title} ' )
prtFlux ('E-P ATM       ', ATM_flx_wemp_ter   , 'f' )
prtFlux ('wbilo ter     ', ATM_flx_wbilo_ter  , 'f' )
prtFlux ('E-P SRF       ', SRF_flx_emp        , 'f' )
prtFlux ('SRF/ATM error ', ATM_flx_wbilo_ter - SRF_flx_emp, 'e', True)
echo ( 'SRF/ATM error {:12.3e} (rel)  '.format ( (ATM_flx_wbilo_ter - SRF_flx_emp)/SRF_flx_emp ) )

echo ('')
echo ( '\n====================================================================================' )
echo ( f'-- RUNOFF fluxes  -- {Title} ' )
RUN_flx_all = RUN_flx_torouting - RUN_flx_river - RUN_flx_coastal
prtFlux ('runoff    ', RUN_flx_runoff      , 'f' ) 
prtFlux ('drainage  ', RUN_flx_drainage    , 'f' ) 
prtFlux ('run+drain ', RUN_flx_torouting   , 'f' ) 
prtFlux ('river     ', RUN_flx_river       , 'f' ) 
prtFlux ('coastal   ', RUN_flx_coastal     , 'f' ) 
prtFlux ('riv+coa   ', RUN_flx_fromrouting , 'f' ) 
prtFlux ('budget    ', RUN_flx_all         , 'f' , small=True)

echo ( '\n------------------------------------------------------------------------------------' )
echo ( f'Water content in routing+lake  -- {Title} ' )
echo ( f'RUN_mas_wat_beg  = {RUN_mas_wat_beg:12.6e} kg | RUN_mas_wat_end = {RUN_mas_wat_end:12.6e} kg ' )
prtFlux ( 'dMass (routing)  ', dRUN_mas_wat+dSRF_mas_lake, 'f', small=True)
prtFlux ( 'Routing error    ', RUN_flx_all+dSRF_mas_lake-dRUN_mas_wat, 'e', small=True )
echo ( 'Routing error : {:12.3e} (rel)'.format ( (RUN_flx_all-dSRF_mas_lake-dRUN_mas_wat)/(dRUN_mas_wat+dSRF_mas_lake) ) )

echo ( '\n------------------------------------------------------------------------------------' )
echo ( f'Water content in routing  -- {Title} ' )
echo ( f'RUN_mas_wat_beg  = {RUN_mas_wat_beg:12.6e} kg | RUN_mas_wat_end = {RUN_mas_wat_end:12.6e} kg ' )
prtFlux ( 'dMass (routing) ', dRUN_mas_wat, 'f', small=True  )
prtFlux ( 'Routing error   ', RUN_flx_all-dRUN_mas_wat, 'e', small=True)
echo ( 'Routing error : {:12.3e} (rel)'.format ( (RUN_flx_all-dRUN_mas_wat)/dRUN_mas_wat ) )

echo ( ' ' )
echo ( f'{Title = }' )

echo ( 'SVN Information' )
for clef in SVN.keys () : echo ( SVN[clef] )
